import os, os.path

from source_list import SourceList

class Component:
    def __init__(self, env, sources = [], headers = []):
        self.env = env

        self.headers = SourceList(env, headers)
        self.sources = SourceList(env, sources)
        self.data = SourceList(env)
        self.other = SourceList(env)

    def lineCount(self):
        return self.headers.lineCount() + self.sources.lineCount()

    def allFiles(self):
        all_files = SourceList(self.env)

        all_files += self.headers
        all_files += self.sources
        all_files += self.data
        all_files += self.other

        return all_files

    HEADER_EXTS = set(['h', 'hh', 'H', 'hpp', 'hxx', 'h++'])
    SOURCE_EXTS = set(['c', 'cc', 'C', 'cpp', 'cxx', 'c++'])

    def ext(self, file):
        return os.path.splitext(str(file))[1][1:]

    def __iadd__(self, a):
        if isinstance(a, Component):
            self.headers += a.headers
            self.sources += a.sources
            self.data += a.data
            self.other += a.other
        else:
            ext = self.ext(a)

            if ext in self.HEADER_EXTS:
                self.headers += a
            elif ext in self.SOURCE_EXTS:
                self.sources += a
            else:
                self.other += a

        return self

    def __add__(A, B):
        sum = Component(A.env)
        
        sum += A
        sum += B

        return sum
