from component import Component
from target import Target
from static_library import StaticLibrary
from source_list import SourceList

class Program(Target):
    def __init__(self, env, name, libs = [], **kwargs):
        Target.__init__(self, env, **kwargs)

        self.name = name
        self.static_libs = []
        self.built = False

        self += libs


    def targetBasename(self):
        s = ''

        s += self.env['PROGPREFIX']
        s += self.name
        s += self.env['PROGSUFFIX']

        return s

    def __iadd__(self, a):
        if type(a) == list or isinstance(a, SourceList):
            for b in a:
                self += b
        elif isinstance(a, StaticLibrary):
            self.static_libs.append(a)
            self += a.sublibs()
        else:
            Component.__iadd__(self, a)

        return self

    def build(self, **kwargs):
        sources = self.sources()
        sources += [str(lib) for lib in self.static_libs]

        self.built = self.env.Program(self.target(), sources, **kwargs)

        if not self.internal:
            self.env.Default(self.built)
