from cStringIO import StringIO
import time
import subprocess

def svnInfo(env):
    if not env.Detect('svn'):
        print 'nosvn'
        return None
    
    proc = subprocess.Popen(['svn', 'info'],
                            stdout = subprocess.PIPE,
                            cwd = str(env.Dir('#')))

    info = {}
    for line in proc.stdout:
        s = line.find(':')
        if s == -1:
            continue
        
        line = (line[:s], line[s+1:])
        line = [x.strip() for x in line]
        line = [x for x in line if len(x) > 0]

        if len(line) != 2:
            continue
        
        info[line[0]] = line[1]

    return info

def getBuildInfo(env, vars):
    f = StringIO()

    f.write('#include "build_info.h"\n');

    # Build options
    nopts = 0
    f.write('static const char *build_opts[] = {')
    for k, v in vars.items():
        f.write('\"%s\", \"%s\",' % (k, v))
        nopts += 1
    f.write('}; ')
    
    f.write('const build_info_t BUILD_INFO = {');


    # Time
    if env['RELEASE']:
        f.write('%d, ' % int(time.time()));
    else:
        f.write('0,')

    # Options
    f.write('%d, ' % nopts);
    f.write('build_opts, ');

    # SVN info
    svn = svnInfo(env)
    if env['RELEASE'] and svn:
        uuid = svn['Repository UUID']
        path = svn['URL'][len(svn['Repository Root']):]
        rev = int(svn['Revision'])
        
        f.write('1, "%s", "%s", %d' % (uuid, path, rev))
    else:
        f.write('0, NULL, NULL, 0');
        
    f.write('};');

    return f.getvalue()

def writeBuildInfo(env, target, vars):
    build_info = getBuildInfo(env, vars)

    env.WriteValue(target, env.Value(build_info))

def writeValue(target, source, env):
    value = ''
    for s in source:
        value += str(s)

    for t in target:
        f = open(str(t), 'w+')
        f.write(value)
        f.close()

def generate(env):
    builder = env.Builder(action = writeValue)
    env.Append(BUILDERS = {'WriteValue': builder})
    
    env.AddMethod(writeBuildInfo, 'BuildInfo')

    
def exists(env):
    return True
