import os, os.path
from SCons.Script import *

from tar_prefix import TarPrefix as Prefix


compression_flags = {
    'gz': '-z',
    'tgz': '-z',
    'bz2': '-j'}

def compressionFlag(tarfile):
    """ Find the appropriate compression flags based on the file extension. """

    ext = os.path.splitext(str(tarfile))[1][1:]
    if ext in compression_flags:
        return compression_flags[ext]
    else:
        return ''
    

def splitSources(sources):
    """ Divide into source files and prefixes. """
    
    # Files
    files = [s for s in sources if not isinstance(s, Prefix)]
    files = [str(s) for s in files]
    files = set(files)

    # Prefixing transform
    prefixes = [s for s in sources if isinstance(s, Prefix)]

    if len(prefixes) == 0:
        prefix = None
        transform = ''
    else:
        prefix = ' '.join([str(p) for p in prefixes])

    return prefix, files
        

def generator(source, target, env, for_signature):
    prefix, files = splitSources(source)
    files = ' '.join(files)
    
    if prefix:
        transform = ' --transform s,^,%s,' % prefix
    else:
        transform = ''

    compression = compressionFlag(target[0])

    return '$TAR $TARFLAGS %s%s -f $TARGET %s' % (compression, transform, files)

def generate(env):
    builder = env.Builder(generator = generator)

    env.Append(BUILDERS = {'PrefixedTar': builder})
        
def exists(env):
    return env.Detect('tar')
