import os
import os.path
import subprocess

from SCons.Script import *

def extension(file):
    return os.path.splitext(str(file))[1][1:]

def compressionOptions(tar):
    """ Figure out the compression option which needs to be passed to
        tar using the extension. """

    ext = extension(tar)       

    if ext in ['gz', 'tgz']:
        return '-z'
    elif ext == 'bz2':
        return '-j'
    elif ext == 'tar':
        return ''
    else:
        raise 1  # TODO: NotImplementedError

def tarContents(tar):
    """ List the files within a tar file. """

    cmd = ['tar',  compressionOptions(tar), '-tf', tar]

    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE).stdout

    contents = []
    for line in proc:
        f = line.strip()
        f = os.path.join(os.path.dirname(tar), f)
        contents.append(f)

    return contents

def generator(source, target, env, for_signature):
    return 'tar -C %s %s -xf $SOURCE' % (os.path.dirname(str(source[0])),
                                         compressionOptions(str(source[0])))

def emitter(target, source, env):
    contents = []

    # Run tar -tf on each
    for tar in source:
        contents.extend(tarContents(str(tar)))

    # Convert to scons Dir or File
    target = []
    for entry in contents:
        if entry[-1] != '/':
            target.append(env.File(entry))

    return target, source

def generate(env):
    # TODO: configure builder to ensure tar exists/is built
    # before running emitter?
    builder = env.Builder(generator = generator)

    env.Append(BUILDERS = {'Untar': builder})

def exists(env):
    return env.Detect('tar')
