#ifndef __BUNDLE_H__
#define __BUNDLE_H__

#include "path.h"

typedef struct bundle bundle_t;

// Constructor/destructor
bundle_t *bundle_create(int dim, int count);
bundle_t *bundle_copy(bundle_t *);
bundle_t *bundle_resize(bundle_t *, int new_count);
void bundle_destroy(bundle_t *);

// Accessors
int bundle_dim(bundle_t *);
int bundle_count(bundle_t *);

// Members
path_t *bundle_get(bundle_t *, int i);
void bundle_set(bundle_t *, int i, path_t *);
void bundle_remove(bundle_t *, int i);
bundle_t *bundle_append(bundle_t *, path_t *);

/**
 * Merges the paths in src into dst
 */
bundle_t *bundle_merge(bundle_t *dst, bundle_t *src);

/**
 * Constructing bundles from paths
 *
 * Samples path uniformly from a tube given the core and radius
 *
 * @arg core The central route of the bundle.
 * @arg radius The radius around the core in which the bundle lives
 * @arg count The number of individual paths in the bundle
 */
bundle_t *bundle_create_uniform(path_t *core, double radius, int count);

/**
 * Find the smallest box containing all points in the bundle.
 */
void bundle_extent(bundle_t *, point_t min, point_t max);

#endif // #ifndef __BUNDLE_H__
