#include "flat_graph.h"

struct flat_graph
{
	int V, E;
};

static inline flat_edge_t *get_edges(flat_graph_t *FG)
{
	intptr_t p;

	p = (intptr_t)FG;
	p += sizeof(flat_graph_t);

	return (flat_edge_t *)p;
}

static inline int *get_vertex_indices(flat_graph_t *FG)
{
	intptr_t p;
	flat_edge_t *e;

	e = get_edges(FG);
	p = (intptr_t)e;
	p += FG->E * sizeof(flat_edge_t);

	return (int *)p;
}

flat_graph_t *flat_graph_create(graph_t *G)
{
	flat_graph_t *FG;
	int i, j, k, d, V, E;
	flat_edge_t *edges;
	edge_t e;
	int *vertex_indices;
	
	V = graph_vertex_count(G);
	E = graph_edge_count(G);

	FG = malloc(sizeof(flat_graph_t)
							+ E * sizeof(flat_edge_t)
							+ (V + 1) * sizeof(int));
	FG->V = V;
	FG->E = E;

	// Fill edges
	edges = get_edges(FG);
	vertex_indices = get_vertex_indices(FG);
	k = 0;
	for(i = 0; i < V; i++) {
		d = graph_vertex_degree(G, i);
		vertex_indices[i] = k;

		for(j = 0; j < d; j++) {
			e = graph_vertex_edge(G, i, j);

			if(e.u < e.v) {
				assert(k < E);
				edges[k].edge = e;
				edges[k].index = k;
				k++;
			}
		}
	}
	assert(k == E);

	vertex_indices[V] = E;

	assert(vertex_indices[0] == 0);

	return FG;
}

void flat_graph_destroy(flat_graph_t *FG)
{
	free(FG);
}

// Accessors
int flat_graph_vertex_count(flat_graph_t *FG)
{
	return FG->V;
}

int flat_graph_edge_count(flat_graph_t *FG)
{
	return FG->E;
}

flat_edge_t *flat_graph_edge(flat_graph_t *FG, int i)
{
	assert(i < FG->E);
	return get_edges(FG) + i;
}

flat_edge_t *flat_graph_vertex_start(flat_graph_t *FG, int i)
{
	assert(i < FG->V);
	return get_edges(FG) + get_vertex_indices(FG)[i];
}


int flat_graph_vertex_degree(flat_graph_t *FG, int i)
{
	int *vertex_indices;

	vertex_indices = get_vertex_indices(FG);

	return vertex_indices[i+1] - vertex_indices[i];
}

int flat_graph_edge_index(flat_graph_t *FG, int u, int v)
{
	int i, end, swap;
	int *vertex_indices;
	flat_edge_t *edges;
	
	edges = get_edges(FG);
	vertex_indices = get_vertex_indices(FG);

	if(u > v) {
		swap = u;
		u = v;
		v = swap;
	}

	end = vertex_indices[u+1];
	for(i = vertex_indices[u]; i < end; i++) {
		if(edges[i].edge.v == v) {
			return i;
		}
	}

	return -1;
}
