#include "flat_line_graph.h"
#include "line_graph_struct.h"
#include "util.h"

typedef struct flat_vertex_entry flat_vertex_entry_t;
struct flat_vertex_entry {
  flat_edge_pair_t *edge_pairs;
	int edge_pair_count;

};

struct flat_line_graph
{
  int vertex_count;
  int edge_pair_count;
  int max_pair_degree;

  flat_vertex_entry_t *vertices;
  flat_edge_pair_t *edge_pairs;
};

static flat_vertex_entry_t *get_flat_vertex_entry(flat_line_graph_t *FG, int v)
{
  assert(v < FG->vertex_count);
  
  return FG->vertices + v;
}


// Constructor/destructor
flat_line_graph_t *flat_line_graph_create(line_graph_t *G)
{
  size_t s;
  flat_line_graph_t *FG;
  intptr_t p;
  int i, j, k;
	vertex_entry_t *v;
  edge_pair_t *ep;

  // Find size
  s = sizeof(flat_line_graph_t);
  s += sizeof(flat_vertex_entry_t) * G->vertex_count;
  s += sizeof(flat_edge_pair_t) * G->edge_pair_count;

  // Allocate + initialize top-level structure
  FG = (flat_line_graph_t *)malloc(s);

  FG->vertex_count = G->vertex_count;
  FG->edge_pair_count = G->edge_pair_count;

  p = (intptr_t)FG;
  p += sizeof(flat_line_graph_t);
  FG->vertices = (flat_vertex_entry_t *)p;

  p += sizeof(flat_vertex_entry_t) * G->vertex_count;
  FG->edge_pairs = (flat_edge_pair_t *)p;

  FG->max_pair_degree = 0;

  k = 0;
  for(i = 0; i < G->vertex_count; i++) {
		v = get_vertex_entry(G, i);

    // Initialize flat vertex
    FG->vertices[i].edge_pairs = FG->edge_pairs + k;
    FG->vertices[i].edge_pair_count = v->edge_pair_count;

    FG->max_pair_degree = MAX(FG->max_pair_degree, v->edge_pair_count);

    // Copy over edge pairs
		for(j = 0; j < v->edge_pair_count; j++) {
			assert(k < G->edge_pair_count);

			FG->edge_pairs[k].edge_pair = v->edge_pairs[j];
			FG->edge_pairs[k].index = k;

      k++;
    }
  }

  return FG;
}

void flat_line_graph_destroy(flat_line_graph_t *FG)
{
  free(FG);
}

// Accessors
int flat_line_graph_vertex_count(flat_line_graph_t *FG)
{
  return FG->vertex_count;
}

int flat_line_graph_total_edge_pair_count(flat_line_graph_t *FG)
{
  return FG->edge_pair_count;
}

int flat_line_graph_max_pair_degree(flat_line_graph_t *FG)
{
  return FG->max_pair_degree;
}

int flat_line_graph_edge_pair_count(flat_line_graph_t *FG, int v)
{
  return get_flat_vertex_entry(FG, v)->edge_pair_count;
}

flat_edge_pair_t *flat_line_graph_edge_pair(flat_line_graph_t *FG, int v, int i)
{
  flat_vertex_entry_t *vertex_entry;

  vertex_entry = get_flat_vertex_entry(FG, v);

  assert(i < vertex_entry->edge_pair_count);

  return vertex_entry->edge_pairs + i;
}

flat_edge_pair_t *flat_line_graph_flat_edge_pair(flat_line_graph_t *FG, int i)
{
  assert(i < FG->edge_pair_count);

  return FG->edge_pairs + i;
}
