#include "graph.h"

typedef struct {
	int degree;
	edge_t *edges;
} vertex_t;

struct graph
{
	int N, E;
	vertex_t **v;
};

graph_t *graph_create(int N)
{
	graph_t *G;
	intptr_t p;
	int i;

	G = (graph_t *)malloc(sizeof(graph_t) + sizeof(vertex_t *) * N);

	G->N = N;
	G->E = 0;
	
	p = (intptr_t)G;
	p += sizeof(graph_t);

	G->v = (vertex_t **)p;
	for(i = 0; i < N; i++) {
		G->v[i] = NULL;
	}

	return G;
}

void graph_destroy(graph_t *G)
{
	int i, N;

	if(G == NULL) {
		return;
	}

	N = G->N;
	for(i = 0; i < N; i++) {
		free(G->v[i]);
	}
	free(G);
}

// Accessors
int graph_vertex_count(graph_t *G)
{
	return G->N;
}

int graph_edge_count(graph_t *G)
{
	return G->E/2;
}

static inline vertex_t *get_vertex(graph_t *G, int i)
{
	assert(i < G->N);

	return G->v[i];
}

int graph_vertex_degree(graph_t *G, int i)
{
	vertex_t *v;

	v = get_vertex(G, i);

	if(v == NULL) {
		return 0;
	}
	else {
		return v->degree;
	}
}

static edge_t *get_edge(graph_t *G, int i, int j)
{
	vertex_t *v;

	v = get_vertex(G, i);

	assert(v != NULL && j < v->degree);

	return v->edges + j;
}

edge_t graph_vertex_edge(graph_t *G, int i, int j)
{
	return *get_edge(G, i, j);
}

int graph_vertex_neighbor(graph_t *G, int i, int j)
{
	return get_edge(G, i, j)->v;
}

void graph_vertex_edge_set_weight(graph_t *G, int i, int j, weight_t w)
{
	get_edge(G, i, j)->weight = w;
}

static void edge_vertex_add(graph_t *G, int i, edge_t *e)
{
	vertex_t *v;
	int d;
	
	assert(i < G->N);

	d = graph_vertex_degree(G, i);

	v = G->v[i] = realloc(G->v[i], sizeof(vertex_t) + sizeof(edge_t) * (d + 1));
	v->degree = d + 1;
	v->edges = (edge_t *)((intptr_t)v + sizeof(vertex_t));

	v->edges[d] = *e;

	G->E++;
}

void graph_edge_add(graph_t *G, edge_t *e)
{
	edge_t r;
	int swap;

	edge_vertex_add(G, e->u, e);

	r = *e;
	swap = r.u;
	r.u = r.v;
	r.v = swap;
	
	edge_vertex_add(G, r.u, &r);
}

bool graph_edge_present(graph_t *G, int u, int v)
{
	int i, d;
	vertex_t *vertex;
	
	vertex = get_vertex(G, u);
	if(vertex == NULL) {
		return false;
	}

	d = vertex->degree;
	for(i = 0; i < d; i++) {
		if(vertex->edges[i].v == v) {
			return true;
		}
	}

	return false;
}
