#ifndef __LINE_GRAPH_H__
#define __LINE_GRAPH_H__

#include "point.h"
#include "graph.h"
#include "bundle.h"

typedef struct line_graph line_graph_t;

typedef struct {
	int common_vertex;
	int neighbors[2];

	double weight;

} edge_pair_t;

// Constructor/destructor
line_graph_t *line_graph_create_from_neighborhoods(
  int dim, int point_count,
	point_t (*get_point)(void *closure_data, int i), void *closure_data,
  double neighborhood_distance);

line_graph_t *line_graph_create_from_graph(graph_t *G);

line_graph_t *line_graph_create(int vertex_count);
//line_graph_t *line_graph_copy(line_graph_t *); // TODO
void line_graph_destroy(line_graph_t *);

// Accessors
int line_graph_vertex_count(line_graph_t *);
int line_graph_total_edge_pair_count(line_graph_t *);

int line_graph_edge_pair_count(line_graph_t *, int vertex);
edge_pair_t *line_graph_edge_pair(line_graph_t *, int vertex, int i);

edge_pair_t *line_graph_add_edge_pair(line_graph_t *, edge_pair_t *);

// Creating edge pair lists from list of edges
void line_graph_vertex_set_neighbors(
  line_graph_t *, int vertex,
  int neighbor_count, int *neighbors);

/**
 * Create all endpoint-to-endpoint paths from the graph.
 *
 * Returns NULL-terminated list of -1-terminated lists of vertices
 */
int **line_graph_paths(line_graph_t *);

/**
 * Threshold a line graph by its weights.
 */
line_graph_t *line_graph_threshold(line_graph_t *, double threshold);

/**
 * Find graph G such that this is its line graph
 */
graph_t *line_graph_unline(line_graph_t *);

#endif // #ifndef __LINE_GRAPH_H__
