#ifndef __NIFTI_H__
#define __NIFTI_H__

#include <nifti1_io.h>

int nifti_filename(const char *filename);

typedef struct {
	// Closure called with explanatory message when error occurs
	void (*error_closure)(void *closure_data, const char *msg);
	void *closure_data;

	/**
	 * Symmetric matrix storage order.
	 *
	 * 'L': Lower-triangular: xx, yx, yy, zx, zy, zz
   * 'U': Upper-triangular: xx, xy, xz, yy, yz, zz
	 */
	char element_order;

} nifti_load_options_t;

extern const nifti_load_options_t nifti_load_options_default;

#include "tensor_field.h"
tensor_field_t *nifti_load_tensor_field(
  const char *filename,
  const nifti_load_options_t *opts,		
	struct nifti_1_header *hdr);

// TODO: save tensor field to nifti file?
// TODO: retrieve nifti_compiled_with_zlib somewhere...
// TODO: init wrapper function for niftilib...

#include "mask.h"
mask_t *nifti_load_mask(
	const char *filename,
  const nifti_load_options_t *opts,	
	struct nifti_1_header *hdr);

void nifti_write_mask(
	const char *filename, mask_t *mask,
	int size[3], float voxdim[3],
	const nifti_load_options_t *opts);

#endif // #ifndef __NIFTI_H__
