#ifndef __OPTIMIZE_H__
#define __OPTIMIZE_H__

#include "line_graph.h"
#include "mask.h"

#include "sparse_matrix.h"

typedef enum {
  SOLVER_DEFAULT,
	SOLVER_NULL,

  SOLVER_GLPK,
  SOLVER_CPLEX
} solver_t;

typedef enum {
	MODEL_ILP1,

	MODEL_QP1,

} model_t;

typedef struct {
	/**
	 * Regularization constants.
	 */
	double lambda, mu;

	/**
	 * Which solver to use.
	 */
	solver_t solver;

	/**
	 * Number of threads the optimizer is encouraged to use.
	 */
	int threads;

	/**
	 * Which of several alternate models to use
	 */
	model_t model;

	/**
	 * Printing status messages.
	 */
	bool verbose;

} optimize_options_t;

extern const optimize_options_t optimize_options_defaults;

// Note: caller must free result
line_graph_t *optimal_contours(optimize_options_t *, line_graph_t *, mask_t *endpoints);



typedef struct {
	int ncols, nrows;
	bool minimize;

	// Columns
	double *obj, *lb, *ub;
	char *ctype;
	char **colnames;

	// Linear constraint matrix
	matrix_list_t *A;
	char **rownames;

	// Linear constraint rhs
	char *sense;
	double *rhs;

	// Quadratic objective matrix
	matrix_list_t *Q;
	
	// Solution
	double *solution;
	
} optimize_problem_t;

optimize_problem_t *optimize_problem_create(
  optimize_options_t *opts,
	int ncols, int nrows, int Annz, int Qnnz);
void optimize_problem_destroy(optimize_problem_t *);

typedef enum {
	OPTIMIZE_NONE = -2,
	OPTIMIZE_ERROR = -1,

	OPTIMIZE_OPTIMAL = 0,
	OPTIMIZE_UNBOUNDED,
	OPTIMIZE_INFEASIBLE

} optimize_status_t;

typedef struct {
	optimize_status_t status;

	double objective;
	double *solution;

} optimize_solution_t;

void optimize_solution_destroy(optimize_solution_t *);

optimize_solution_t *optimize(optimize_options_t *, optimize_problem_t *prob);

#endif // #ifndef __OPTIMIZE_H__
