#ifndef __TEND_H__
#define __TEND_H__

#include "tensor_field.h"
#include "mask.h"
#include "bundle.h"

/**
 * Configuration options for the TEND algorithm
 */
typedef struct {
	/**
	 * Weighting on the principal eigenvector.
	 */
	double f;

	/**
	 * Weighting on the TEND update vector from this step.
	 */
	double g;

	/**
	 * The distance to move in each update step.
	 */
	double step;

	/**
	 * Curvature limit above which to terminate the tract.
	 *
	 * Approximated by the angle between v_in and v_out.
	 */
	double curvature_limit;

	/**
	 * Minimum FA limit below which to terminate the tract.
	 */
	double FA_limit;

} tend_opts_t;

extern const tend_opts_t tend_opts_default;

/**
 * Perform TEND tracking.
 *
 * @arg tf The tensor field giving the orientatin information.
 * @arg roi Sample starting points from the given ROI.
 * @arg tracts The number of times to start the TEND tracing.
 * @return A list of tracts produced by the tracing.
 */
bundle_t *tend(tensor_field_t *tf, mask_t *roi, int tracts, const tend_opts_t *opts);

#endif // #ifndef __TEND_H__
