#ifndef __TENSOR_H__
#define __TENSOR_H__

#include "point.h"

typedef coord_t tensor_elt_t;
typedef struct tensor tensor_t;

size_t tensor_size(int dim);

// Constructor/destructor
tensor_t *tensor_create(int dim);
tensor_t *tensor_create_at(int dim, void *location);
tensor_t *tensor_copy(tensor_t *);
void tensor_destroy(tensor_t *);

void tensor_assign(tensor_t *dst, tensor_t *src);

int tensor_dim(tensor_t *T);

tensor_elt_t tensor_elt_get(tensor_t *T, int i, int j);
void tensor_elt_set(tensor_t *T, int i, int j, tensor_elt_t v);

// Calculates T * v
void tensor_mult(point_t prod, tensor_t *T, point_t v);

// Calculates v^T * T * v
coord_t tensor_mult2(tensor_t *T, point_t v);

// Assigns the tensor to be a dyadic tensor of the given vector
void tensor_dyadic(tensor_t *, point_t v);

// Returns as gsl matrix
// Note: caller must free result
gsl_matrix *tensor_as_gsl_matrix(tensor_t *);

/**
 * Multiply the tensor by a constant
 */
void tensor_scale(tensor_t *, tensor_elt_t s);

/**
 * Add one tensor into the other
 */
void tensor_add(tensor_t *, tensor_t *);

// Normalize so lambda1 = 1.0
void tensor_normalize(tensor_t *);

// Sets a tensor to zero
void tensor_zero(tensor_t *T);

// Sets a tensor to the identity
void tensor_identity(tensor_t *T);

// Measures on a tensor
tensor_elt_t tensor_trace(tensor_t *T);
tensor_elt_t tensor_mean_diffusivity(tensor_t *T);

// I/O
void tensor_print(tensor_t *T);

void tensor_printf(tensor_t *T, int lower);
void tensor_fprintf(FILE *, tensor_t *T, int lower);
int tensor_snprintf(size_t buflen, char *buf, tensor_t *T, int lower);

void tensor_printf_fmt(tensor_t *T, const char *coord_fmt, int lower);
void tensor_fprintf_fmt(FILE *, tensor_t *T, const char *coord_fmt, int lower);
int tensor_snprintf_fmt(
  size_t buflen, char *buf,
  tensor_t *T,
  const char *coord_fmt, int lower);


#endif // #ifndef __TENSOR_H__
