#ifndef __TENSOR_EIGENS_H__
#define __TENSOR_EIGENS_H__

#include "tensor.h"

typedef struct tensor_eigens tensor_eigens_t;

// Constructor/destructor
size_t tensor_eigens_size(int dim);

tensor_eigens_t *tensor_eigens_create(int dim);
tensor_eigens_t *tensor_eigens_create_at(int dim, void *location);
tensor_eigens_t *tensor_eigens_copy(tensor_eigens_t *);
void tensor_eigens_destroy(tensor_eigens_t *);

void tensor_eigens_assign(tensor_eigens_t *dst, tensor_eigens_t *src);

// Accessors
// Note: these are sorted in descending order
coord_t tensor_eigenvalue(tensor_eigens_t *e, int i);
point_t tensor_eigenvector(tensor_eigens_t *e, int i);

// Perform eigendecomposition
void tensor_eigens_calc_lapack(tensor_t *T, tensor_eigens_t *);
void tensor_eigens_calc_gsl(tensor_t *T, tensor_eigens_t *);

#define tensor_eigens_calc tensor_eigens_calc_gsl

// Scaled principal eigenvector
void tensor_principal_vector(tensor_t *T, point_t v);

// Print eigensystem
void tensor_eigens_print(tensor_eigens_t *);

/**
 * Get the eigensystem for a given tensor.
 *
 * @note Caller does NOT need to free result.
 */
tensor_eigens_t *tensor_eigens_get(tensor_t *);

// Measures over eigensystem
coord_t tensor_relative_anisotropy(tensor_eigens_t *e);
coord_t tensor_fractional_anisotropy(tensor_eigens_t *e);
coord_t tensor_relative_volume_ratio(tensor_eigens_t *e);

#endif // #ifndef __TENSOR_EIGENS_H__
