#include "bspline.h"

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_statistics.h>

int main(int argc, char **argv)
{
  const int length = 64, dim = 2, N = 1e3;
  const double sigma = 0.05;

  path_t *path;
  int i, j;
  double x, y, t;
  gsl_rng *rand;
  point_t p;
  bspline_t *spline;
  coord_t q[dim];

  FILE *path_out, *spline_out;

  assert(argc == 3);
  
  gsl_rng_env_setup();
  rand = gsl_rng_alloc(gsl_rng_default);

  // Initialize path
  path = path_create(dim, length);
  for(i = 0; i < length; i++) {
    t = 10.0 / (length - 1) * i;

    x = sin(t);
    x += gsl_ran_gaussian(rand, sigma * x);

    y = cos(t) * exp(-0.1 * t);
    y += gsl_ran_gaussian(rand, sigma * y);

    p = path_get(path, i);
    p[0] = x;
    p[1] = y;
  }

  // Create spline
  printf("Creating spline...\n");
  spline = bspline_create(path, 4);

  printf("Output...\n");
  // Output paths
  path_out = fopen(argv[1], "w+");
  for(i = 0; i < length; i++) {
    p = path_get(path, i);

    for(j = 0; j < dim; j++) {
      fprintf(path_out, "%.10le ", p[j]);
    }
    fputc('\n', path_out);
  }
  fclose(path_out);

  // Output spline
  spline_out = fopen(argv[2], "w+");
  for(i = 0; i < N; i++) {
    t = (double)i / (N - 1);
    bspline_calculate(spline, t, q);

    for(j = 0; j < dim; j++) {
      fprintf(path_out, "%.10le ", q[j]);
    }
    fputc('\n', path_out);
  }
  fclose(spline_out);

  bspline_destroy(spline);
  path_destroy(path);
  gsl_rng_free(rand);

  return 0;
}
