#ifndef __WEIGHT_VECTOR_H__
#define __WEIGHT_VECTOR_H__

#include "point.h"
#include "weights.h"
#include "line_graph.h"
#include "tensor_field.h"

void canonical_point_pair(int dim, point_t p[2], point_t normed[2]);

////////////////////////// weight_vector //////////////////////////

// Vectors of the spline weights for a given pair of points
typedef struct weight_vector weight_vector_t;

weight_vector_t *weight_vector_create(int dim, int angle_count);
void weight_vector_destroy(weight_vector_t *w);

int weight_vector_dim(weight_vector_t *w);
point_t weight_vector_pos(weight_vector_t *w, int i);
int weight_vector_angle_count(weight_vector_t *w);
int weight_vector_total_angles(weight_vector_t *);

double *weight_vector_elt(weight_vector_t *w, int *i);

// TODO: interpolation?



////////////////////////// weight_vector_array //////////////////////////
typedef struct weight_vector_array weight_vector_array_t;
struct weight_vector_array {
	int length;
	weight_vector_t **w;
};

weight_vector_array_t *weight_vector_array_create(int len);
weight_vector_array_t *weight_vector_array_resize(weight_vector_array_t *, int new_len);
void weight_vector_array_destroy(weight_vector_array_t *);
weight_vector_array_t *weight_vector_array_append(weight_vector_array_t *, weight_vector_t *);

int weight_vector_array_total_weights(weight_vector_array_t *wva);

/**
 * Find the weight vector for a point pair.
 *
 * Gets a point pair with $\ell_\infty$ distance less than tol.
 * Return NULL if none exists.
 */
weight_vector_t *weight_vector_array_get(
  weight_vector_array_t *,
	length_t tol, int dim, point_t pos[2]);

/**
 * Create a vector array based on the triplets in a given line graph
 */
weight_vector_array_t *weight_vector_array_for_line_graph(line_graph_t *L, tensor_field_t *tf, length_t tol, int angle_count);

/**
 * Fill in the weights for a given weight vector array.
 */
void weight_vector_array_calculate(
  weight_vector_array_t *wva,
  int thread_count,
  double (*weight_closure)(void *data, int dim, point_t pos[3], point_t direction[3]), void *closure_data);

////////////////////////// weights //////////////////////////

typedef struct weight_vector_calc_opts weight_vector_calc_opts_t;
struct weight_vector_calc_opts {
	weight_vector_array_t *weights;
	length_t tol;
};

extern const weight_vector_calc_opts_t weight_vector_calc_opts_defaults;

double weight_vector_calc(
  int dim, weight_options_t *,
	point_t points[3], tensor_t *tensors[3]);

#endif // #ifndef __WEIGHT_VECTOR_H__
