#ifndef __IMAGE_BINARY_H__
#define __IMAGE_BINARY_H__

typedef uint64_t bit_group_t;

typedef struct {
	unsigned int width, stride, height;

	/*
	 * The pixel data, one bit ber pixel
	 */
	bit_group_t *pixels;

} image_binary_t;

#define GROUP_SIZE (8 * sizeof(bit_group_t))

/*
 * Create an uninitialized image.
 */
image_binary_t *image_binary_create(
  unsigned int width,
	unsigned int height);

/*
 * Free all memory associated with the image
 */
void image_binary_destroy(image_binary_t *);

static inline bit_group_t bit_mask(int k)
{
	return (bit_group_t)1 << k;
}

static inline int binary_get_pixel(image_binary_t *b, int i, int j)
{
	bit_group_t pixel_group;

	pixel_group = b->pixels[i * b->stride + j / GROUP_SIZE];
	pixel_group &= bit_mask(j % GROUP_SIZE);

	return pixel_group != 0;
}

static inline void binary_set_pixel(image_binary_t *b, int i, int j, int v)
{
	bit_group_t *pixel_group, mask;
	
	pixel_group = &(b->pixels[i * b->stride + j / GROUP_SIZE]);
	mask = bit_mask(j % GROUP_SIZE);

	if(v) {
		*pixel_group |= mask;
	}
	else {
		*pixel_group &= ~mask;
	}
}

/**
 * Builds a binary image based on a condition.
 */
image_binary_t *image_binary_condition(
  int width, int height,
	int (*condition)(void *data, int i, int j), void *data);

#include "grey.h"

/*
 * Threshold a greyscale image to a binary one
 *
 * Sets pixels in binary image to whether the corresponding pixel
 * in the greyscale image is >= tau
 */
image_binary_t *image_threshold(image_grey_t *grey, uint8_t tau);

/*
 * Move a binary image back to black-white greyscale
 */
image_grey_t *image_binary_to_grey(image_binary_t *);

#endif // #ifndef __IMAGE_BINARY_H__
