#include "blur.h"
#include "float.h"
#include "conv.h"

image_float_t *gaussian_kernel(double sigma, unsigned int size)
{
	image_float_t *K;
	int i, j, center;
	double p;

	K = image_float_create(size, size);
	center = size/2;

	for(i = 0; i < size; i++) {
		for(j = 0; j < size; j++) {
			p = gsl_ran_bivariate_gaussian_pdf(i - center, j - center, sigma, sigma, 0);
			*float_pixel(K, i, j) = (float)p;
		}
	}

	return K;
}

void gaussian_blur(image_float_t *I, double sigma, image_float_t *out)
{
	image_float_t *K;

	K = gaussian_kernel(sigma, ceil(4*sigma));
	convolve(I, K, out);

	image_float_destroy(K);
}
