#include "conv.h"

static inline int bound(int min, int x, int max)
{
	if(x < min) {
		return min;
	}
	if(x > max) {
		return max;
	}
	return x;
}

void convolve(image_float_t *I, image_float_t *K, image_float_t *out)
{
	assert(I->width == out->width && I->height == out->height);

	int width_I, height_I, width_K, height_K;
  int i_I, j_I, i_K, j_K, i_out, j_out;
	
	float sum, p_I, p_K;

	// Grab dimensions
	width_I = I->width;
	height_I = I->height;

	width_K = K->width;
	height_K = K->height;

	// Iterate over each pixel of output
	for(i_out = 0; i_out < height_I; i_out++) {
		for(j_out = 0; j_out < width_I; j_out++) {

			sum = 0;

			// Iterate over each kernel pixel
			for(i_K = 0; i_K < height_K; i_K++) {
				for(j_K = 0; j_K < width_K; j_K++) {
					// Get the indexes into the source image we will use
					// This bounding is equivalent to padding A by repeating the
					// edge pixels
					i_I = bound(0, i_out + i_K - height_K/2, height_I - 1);
					j_I = bound(0, j_out + j_K - width_K/2,  width_I  - 1);

					p_I = *float_pixel(I, i_I, j_I);
					p_K = *float_pixel(K, i_K, j_K);

					sum += p_I * p_K;
				}
			}
			
			*float_pixel(out, i_out, j_out) = sum;
		}
	}
}
