#include "float.h"


#define STRIDE_MULTIPLE (4)

unsigned int image_float_stride(unsigned int width)
{
	int stride;

	// Make the stride the least multiple of STRIDE_MULTIPLE greater than width
	if(width % STRIDE_MULTIPLE == 0) {
		stride = width;
	}
	else {
		stride = width + STRIDE_MULTIPLE - (width % STRIDE_MULTIPLE);
	}

	return stride;
}

size_t image_float_data_size(unsigned int width, unsigned int height)
{
	return height * image_float_stride(width) * sizeof(double);
}

image_float_t *image_float_create(
  unsigned int width,
	unsigned int height)
{
	image_float_t *I;
	unsigned int stride;
	size_t s;
	intptr_t p;

	stride = image_float_stride(width);

	// Find the size of the image struct and it's pixel data
	s = sizeof(image_float_t);
	s += image_float_data_size(width, height);

	// Allocate space for all of these
	I = (image_float_t *)malloc(s);
	if(I == NULL) {
		fprintf(stderr, "malloc failure\n");
		return NULL;
	}

	// Find the start of pixel data
	p = (intptr_t)I + sizeof(image_float_t);

	// Initialize image struct
	I->width = width;
	I->stride = stride;
	I->height = height;
	I->pixels = (double *)p;

	return I;
}

image_float_t *image_float_clone(image_float_t *I)
{
	image_float_t *clone;
	size_t s;

	s = image_float_data_size(I->width, I->height) + sizeof(image_float_t);

	clone = (image_float_t *)malloc(s);
	memcpy(clone, I, s);
	clone->pixels = (double *)((intptr_t)clone + sizeof(image_float_t));

	return clone;
}

void image_float_destroy(image_float_t *I)
{
	free(I);
}

/*
 * Convert grayscale image to use floating-point values
 */
image_float_t *image_grey_to_float(image_grey_t *G)
{
	int i, j, width, height;
	image_float_t *F;

	width = G->width;
	height = G->height;
	F = image_float_create(width, height);

	for(i = 0; i < height; i++) {
		for(j = 0; j < width; j++) {
			*float_pixel(F, i, j) = *grey_pixel(G, i, j) / 255.0;
		}
	}

	return F;
}

image_grey_t *image_float_to_grey(image_float_t *F)
{
	image_grey_t *G;
	int i, j, width, height;
	double f, min, max;
	uint8_t g;

	width = F->width;
	height = F->height;
	G = image_grey_create(width, height);

	// Find minimum and maximum
	min = max = *float_pixel(F, 0, 0);

	for(i = 0; i < height; i++) {
		for(j = 0; j < width; j++) {
			f = *float_pixel(F, i, j);

			if(f < min) {
			  min = f;
			}
			if(f > max) {
				max = f;
			}

		}
	}

	for(i = 0; i < height; i++) {
		for(j = 0; j < width; j++) {
			f = *float_pixel(F, i, j);
			g = (uint8_t)((f - min) / (max - min) * 255 + 0.5);

			*grey_pixel(G, i, j) = g;
		}
	}

	return G;
}
