#ifndef __IMAGE_FLOAT_H__
#define __IMAGE_FLOAT_H__

// A greyscale image using double-precision floats

typedef struct {
	unsigned int width, stride, height;

	/*
	 * The pixel data of the image.
	 *
	 * Single-precision floating-point values
	 */
	double *pixels;

} image_float_t;

/*
 * Retrieve the pixel at (i, j) from the image
 */
static inline double *float_pixel(image_float_t *I, unsigned int i, unsigned int j)
{
	assert(i < I->height);
	assert(j < I->width);

	return I->pixels + (I->stride * i + j);
}

/*
 * Create an uninitialized image.
 */
image_float_t *image_float_create(
  unsigned int width,
	unsigned int height);

image_float_t *image_float_clone(image_float_t *);

/*
 * The space allocated to each row of a float image
 */
unsigned int image_float_stride(unsigned int width);

/*
 * Free all memory associated with the image
 */
void image_float_destroy(image_float_t *);

/*
 * How much data a given float requires
 */
size_t image_float_data_size(unsigned int width, unsigned int height);

#include "grey.h"

/*
 * Convert greyscale image to use floating-point values
 */
image_float_t *image_grey_to_float(image_grey_t *);

/*
 * Scale and round floating-point values to get a normal greyscale image
 */
image_grey_t *image_float_to_grey(image_float_t *);

#endif // #ifndef __IMAGE_FLOAT_H__
