#include "grey.h"

#define STRIDE_MULTIPLE (16)

image_grey_t *image_grey_create(
  unsigned int width,
	unsigned int height)
{
	image_grey_t *I;
	unsigned int stride;
	size_t s;
	intptr_t p;

	// Make the stride the least multiple of STRIDE_MULTIPLE greater than width
	if(width % STRIDE_MULTIPLE == 0) {
		stride = width;
	}
	else {
		stride = width + STRIDE_MULTIPLE - (width % STRIDE_MULTIPLE);
	}

	// Find the size of the image struct and it's pixel data
	s = sizeof(image_grey_t);
	s += height * stride * sizeof(uint8_t);

	// Allocate space for all of these
	I = (image_grey_t *)malloc(s);
	if(I == NULL) {
		fprintf(stderr, "malloc failure\n");
		return NULL;
	}

	// Find the start of pixel data
	p = (intptr_t)I + sizeof(image_grey_t);

	// Initialize image struct
	I->width = width;
	I->stride = stride;
	I->height = height;
	I->pixels = (uint8_t *)p;

	return I;
}

void image_grey_destroy(image_grey_t *I)
{
	free(I);
}

image_grey_t *image_grey_clone(image_grey_t *G)
{
	image_grey_t *clone;

	clone = image_grey_create(G->width, G->height);
	memcpy(clone->pixels, G->pixels, G->stride * G->height);

	return clone;
}

void image_grey_histogram(image_grey_t *grey, unsigned int histogram[256])
{
	unsigned int width, height, i, j;

	for(i = 0; i < 256; i++) {
		histogram[i] = 0;
	}

	width = grey->width;
	height = grey->height;

	for(i = 0; i < height; i++) {
		for(j = 0; j < width; j++) {
			histogram[*grey_pixel(grey, i, j)]++;
		}
	}
}

uint8_t image_grey_median(image_grey_t *grey)
{
	unsigned int histogram[256], i;
	int left, right;

	image_grey_histogram(grey, histogram);

	left = right = 0;
	for(i = 0; i < 256; i++) {
		right += histogram[i];
	}

	i = 0;
	while(left < right && i < 256) {
		left += histogram[i];
		right -= histogram[i];
		i++;
	}

	return i;
}

image_grey_t *image_rgb_to_grey(image_rgb_t *rgb)
{
	unsigned int width, height, i, j;
	image_grey_t *grey;

	uint8_t *p;
	unsigned int g;

	width = rgb->width;
	height = rgb->height;

	grey = image_grey_create(width, height);

	for(i = 0; i < height; i++) {
		for(j = 0; j < width; j++) {
			p = rgb_pixel(rgb, i, j);
			g = (54 * p[0] + 183 * p[1] + 19 * p[2]) / 256;
			*grey_pixel(grey, i, j) = (uint8_t)g;
		}
	}

	return grey;
}

image_rgb_t *image_grey_to_rgb(image_grey_t *grey)
{
	unsigned int i, j, c, width, height;
	uint8_t *p, g;
	image_rgb_t *rgb;

	width = grey->width;
	height = grey->height;

	rgb = image_rgb_create(width, height);

	for(i = 0; i < height; i++) {
		for(j = 0; j < width; j++) {
			g = *grey_pixel(grey, i, j);
			p = rgb_pixel(rgb, i, j);

			for(c = 0; c < 3; c++) {
				p[c] = g;
			}
			p[3] = 255;
		}
	}

	return rgb;
}
