#ifndef __IMAGE_GREY_H__
#define __IMAGE_GREY_H__

typedef struct {
	unsigned int width, stride, height;

	/*
	 * The pixel data of the image.
	 *
	 * Simple 8-bit intensity values
	 */
	uint8_t *pixels;

} image_grey_t;

/*
 * Retrieve the pixel at (i, j) from the image
 */
static inline uint8_t *grey_pixel(image_grey_t *I, unsigned int i, unsigned int j)
{
	assert(i < I->height);
	assert(j < I->width);

	return I->pixels + (I->stride * i + j);
}

/*
 * Create an uninitialized image.
 */
image_grey_t *image_grey_create(
  unsigned int width,
	unsigned int height);

/*
 * Free all memory associated with the image
 */
void image_grey_destroy(image_grey_t *);

/*
 * Copy image to a new instance
 */
image_grey_t *image_grey_clone(image_grey_t *);

/*
 * Calculate the histogram of pixel values
 */
void image_grey_histogram(image_grey_t *grey, unsigned int histogram[256]);

/*
 * Calculate the median pixel value across the image
 */
uint8_t image_grey_median(image_grey_t *grey);

#include "rgb.h"

/*
 * Convert an RGB image to greyscale.
 *
 * Uses formula from Ward paper.
 */
image_grey_t *image_rgb_to_grey(image_rgb_t *rgb);

/*
 * Convert a grayscale image to RGB
 */
image_rgb_t *image_grey_to_rgb(image_grey_t *);

#endif // #ifndef __IMAGE_GREY_H__
