#include "interpolate.h"

static inline int bound(int min, int x, int max)
{
	if(x < min) {
		return min;
	}
	if(x > max) {
		return max;
	}
	return x;
}

// Perform bilinear interpolation to determine a subpixel value
double interpolate(image_float_t *image, double in_y, double in_x)
{
	double x[2], y[2];
	double I[2][2];
	int i[2], j[2], a, b;
	double x_interp[2], interp;

	// Perform rounding + bounding
	x[0] = floor(in_x);
	x[1] = ceil(in_x);
	y[0] = floor(in_y);
	y[1] = ceil(in_y);

	for(a = 0; a < 2; a++) {
		i[a] = bound(0, (int)y[a], image->height - 1);
		j[a] = bound(0, (int)x[a], image->width - 1);
	}

	//printf("interp (i_low, i_up), (j_low, j_up) = %d, %d, %d, %d\n",
	//			 i[0], i[1], j[0], j[1]);

	// Retrieve intensities
	for(a = 0; a < 2; a++) {
		for(b = 0; b < 2; b++) {
			I[a][b] = *float_pixel(image, i[a], j[b]);
		}
	}

	// interpolatin along x
	for(a = 0; a < 2; a++) {
		if(j[0] == j[1]) {
			x_interp[a] = I[a][0];
		}
		else {
			assert(j[1] - j[0] == 1);

			x_interp[a] = (in_x - x[0]) * I[a][0] + (x[1] - in_x) * I[a][1];
		}
	}

	if(i[0] == i[1]) {
		interp = x_interp[0];
	}
	else {
		interp = (in_y - y[0]) * x_interp[0] + (y[1] - in_y) * x_interp[1];
	}

	return interp;
}
