#include "jpeg.h"

#include <jpeglib.h>

image_rgb_t *jpeg_load(const char *filename)
{
  struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerr;
  JSAMPARRAY buffer;

	FILE *f;
	image_rgb_t *I;
	int i, j, k, width, height, stride, channels;
	uint8_t *p;

	f = fopen(filename, "rb");
	if(f == NULL) {
		return NULL;
	}

	// Set up
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cinfo);
	jpeg_stdio_src(&cinfo, f);

	// Read header + allocate buffer
	jpeg_read_header(&cinfo, 1);

	// Start reading + allocate buffer
	jpeg_start_decompress(&cinfo);
	width = cinfo.output_width;
	height = cinfo.output_height;
	channels = cinfo.output_components;
	stride = width * channels;

	I = image_rgb_create(width, height);

	// Read image
	buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr)&cinfo, JPOOL_IMAGE, stride, 1);

	for(i = 0; cinfo.output_scanline < height; i++) {
		//printf("i = %d, height = %d\n", i, height);
		jpeg_read_scanlines(&cinfo, buffer, 1);

		// Copy over to our image array
		for(j = 0; j < width; j++) {
			p = rgb_pixel(I, i, j);

			for(k = 0; k < 3; k++) {
				// TODO: Choose based on color space
				if(channels == 3) {
					p[k] = buffer[0][j * channels + k];
				}
				else {
					assert(channels == 1);
					p[k] = buffer[0][j];
				}
			}
			p[3] = 255;
		}
	}

  // End
	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);

	fclose(f);

	return I;
}

void jpeg_write(const char *filename, image_rgb_t *I)
{
  struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
  JSAMPARRAY buffer;

	FILE *f;
	int i, j, k, width, height;
	uint8_t *p;

	uint8_t row[I->width * 3];
	JSAMPROW row_pointer = (JSAMPLE *)&row;
	
	f = fopen(filename, "wb");

	// Set up
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_compress(&cinfo);
	jpeg_stdio_dest(&cinfo, f);

	// Set properties
	width = cinfo.image_width = I->width;
	height = cinfo.image_height = I->height;
	cinfo.input_components = 3;
	cinfo.in_color_space = JCS_RGB;

	jpeg_set_defaults(&cinfo);

	// Start writing
	jpeg_start_compress(&cinfo, 1);

	for(i = 0; i < height; i++) {
		for(j = 0; j < width; j++) {
			p = rgb_pixel(I, i, j);

			for(k = 0; k < 3; k++) {
				row[j * 3 + k] = p[k];
			}
		}

		jpeg_write_scanlines(&cinfo, &row_pointer, 1);
	}

	jpeg_finish_compress(&cinfo);
	fclose(f);
	jpeg_destroy_compress(&cinfo);
}
