#include "png.h"

#include <png.h>

image_rgb_t *png_load(const char *filename)
{
	FILE *f;
	png_structp png_ptr;
	png_infop info_ptr;
	png_uint_32 width, height;
	int bit_depth, color_type, interlace_type;
	image_rgb_t *I;
	
	// Initialize the read
	f = fopen(filename, "rb");
	if(f == NULL) {
		return NULL;
	}

	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	if(png_ptr == NULL) {
		fclose(f);
		return NULL;
	}

	info_ptr = png_create_info_struct(png_ptr);
	if(info_ptr == NULL) {
		fclose(f);
		png_destroy_read_struct(&png_ptr, NULL, NULL);
		return 0;
	}

	png_init_io(png_ptr, f);

	// Grab basic image info
	png_read_info(png_ptr, info_ptr);

	png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type,
							 &interlace_type, int_p_NULL, int_p_NULL);

	// Normalize image format
	if(color_type == PNG_COLOR_TYPE_PALETTE) {
		png_set_palette_to_rgb(png_ptr);
	}

	if(color_type == PNG_COLOR_TYPE_GRAY
		 || color_type == PNG_COLOR_TYPE_RGB) {
		png_set_add_alpha(png_ptr, 255, PNG_FILLER_AFTER);
	}

	if(color_type == PNG_COLOR_TYPE_GRAY
		 || color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
		png_set_gray_to_rgb(png_ptr);
	}

	if(bit_depth == 16) {
		png_set_strip_16(png_ptr);
	}
	else if(bit_depth < 8) {
		png_set_expand(png_ptr);
	}

	// Initialize our image
	I = image_rgb_create(width, height);
	
	// Read in the pixel data
	png_bytep row_pointers[height];
	png_uint_32 y;

	for(y = 0; y < height; y++) {
		row_pointers[y] = rgb_pixel(I, y, 0);
	}

	png_read_image(png_ptr, row_pointers);

	// Cleanup
	png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
	fclose(f);

	return I;
}

void png_write(const char *filename, image_rgb_t *rgb)
{
   FILE *f;
   png_structp png_ptr;
   png_infop info_ptr;

	 int i;

	 // Initialize
	 f = fopen(filename, "wb");
	 if(f == NULL) {
		 return;
	 }

	 png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	 if(png_ptr == NULL) {
		 fclose(f);
		 return;
	 }

	 info_ptr = png_create_info_struct(png_ptr);
   if(info_ptr == NULL)
   {
      fclose(f);
      png_destroy_write_struct(&png_ptr,  NULL);
      return;
   }

	 png_init_io(png_ptr, f);

	 png_set_IHDR(png_ptr, info_ptr, rgb->width, rgb->height, 8, PNG_COLOR_TYPE_RGBA,
								PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,
								PNG_FILTER_TYPE_DEFAULT);


	 png_bytep row_pointers[rgb->height];

	 for(i = 0; i < rgb->height; i++) {
		 row_pointers[i] = rgb_pixel(rgb, i, 0);
	 }

	 png_set_rows(png_ptr, info_ptr, row_pointers);

	 // Write
	 png_write_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL);

	 // Cleanup
   png_destroy_write_struct(&png_ptr, &info_ptr);
	 fclose(f);
}
