#include "rgb.h"

#define STRIDE_MULTIPLE (64)

image_rgb_t *image_rgb_create(
  unsigned int width,
	unsigned int height)
{
	image_rgb_t *I;
	unsigned int stride;
	size_t s;
	intptr_t p;

	// Make the stride the least multiple of STRIDE_MULTIPLE greater than width
	if(width % STRIDE_MULTIPLE == 0) {
		stride = width;
	}
	else {
		stride = width + STRIDE_MULTIPLE - (width % STRIDE_MULTIPLE);
	}

	// Find the size of the image struct and it's pixel data
	s = sizeof(image_rgb_t);
	s += height * stride * 4 * sizeof(uint8_t);
	
	// Allocate space for all of these
	I = (image_rgb_t *)malloc(s);

	if(I == NULL) {
		fprintf(stderr, "malloc failure\n");
		return NULL;
	}

	// Find the start of pixel data
	p = (intptr_t)I + sizeof(image_rgb_t);

	// Initialize image struct
	I->width = width;
	I->stride = stride;
	I->height = height;
	I->pixels = (uint8_t *)p;

	return I;
}

image_rgb_t *image_rgb_clone(image_rgb_t *I)
{
	image_rgb_t *clone;

	clone = image_rgb_create(I->width, I->height);
	assert(clone->stride == I->stride);
	memcpy(clone->pixels, I->pixels, I->stride * I->height * 4);

	return clone;
}

void image_rgb_destroy(image_rgb_t *I)
{
	free(I);
}

void image_rgb_set_background(image_rgb_t *I, const uint8_t color[4])
{
	unsigned int i, j, k, width, height;
	uint8_t *pixel;
	
	width = I->width;
	height = I->height;

	for(i = 0; i < height; i++) {
		for(j = 0; j < width; j++) {
			pixel = rgb_pixel(I, i, j);

			for(k = 0; k < 4; k++) {
				pixel[k] = color[k];
			}
		}
	}
}

static const uint8_t ZERO[4];

void image_rgb_set_zero(image_rgb_t *I)
{
	image_rgb_set_background(I, ZERO);
}
