#ifndef __IMAGE_RGB_H__
#define __IMAGE_RGB_H__

typedef struct {
	unsigned int width, stride, height;

	/*
	 * The pixel data of the image.
	 *
	 * Pixels stored in (R, G, B) format in row-major format with:
	 *   - 1 byte per channel
	 *   - 4 bytes per pixel (last unused)
   *   - stride bytes per row (anything beyond 4*width unused)
	 *
	 */
	uint8_t *pixels;

} image_rgb_t;

/*
 * Retrieve the pixel at (i, j) from the image
 */
static inline uint8_t *rgb_pixel(image_rgb_t *I, unsigned int i, unsigned int j)
{
	assert(i < I->height);
	assert(j < I->width);

	return I->pixels + (I->stride * i + j) * 4;
}

static inline uint8_t red_pixel(image_rgb_t *I, unsigned int i, unsigned int j)
{
	return rgb_pixel(I, i, j)[0];
}

static inline uint8_t green_pixel(image_rgb_t *I, unsigned int i, unsigned int j)
{
	return rgb_pixel(I, i, j)[1];
}

static inline uint8_t blue_pixel(image_rgb_t *I, unsigned int i, unsigned int j)
{
	return rgb_pixel(I, i, j)[2];
}

/*
 * Create an uninitialized image.
 */
image_rgb_t *image_rgb_create(
  unsigned int width,
	unsigned int height);

/*
 * Clone an image
 */
image_rgb_t *image_rgb_clone(image_rgb_t *);

/*
 * Free all memory associated with the image
 */
void image_rgb_destroy(image_rgb_t *);

/*
 * Create a background of the specified color
 */
void image_rgb_set_background(image_rgb_t *, const uint8_t color[4]);

/*
 * Create a zero, transparent image
 */
void image_rgb_set_zero(image_rgb_t *);

#endif // #ifndef __IMAGE_RGB_H__
