#include "bundle_io.h"

void bundle_write(FILE *fp, bundle_t *b)
{
	path_t *path;
	point_t p;
	int i, j, k, count, true_count, length, dim;

	dim = bundle_dim(b);
	count = bundle_count(b);

	// Count non-null entries
	true_count = 0;
	for(i = 0; i < count; i++) {
		if(bundle_get(b, i) != NULL) {
			true_count++;
		}
	}

	fprintf(fp, "dim = %d\n", dim);
	fprintf(fp, "count = %d\n", true_count);

	for(i = 0; i < count; i++) {
		path = bundle_get(b, i);

		if(path == NULL) {
			continue;
		}

		assert(dim == path_dim(path));
		length = path_length(path);

		fprintf(fp, "length = %d\n", length);

		for(j = 0; j < length; j++) {
			p = path_get(path, j);

			for(k = 0; k < dim; k++) {
				fprintf(fp, "%.10e ", p[k]);
			}
			fputc('\n', fp);
		}
		fputc('\n', fp);
	}
}

void bundle_write_file_name(const char *filename, bundle_t *b)
{
	FILE *fp;

	fp = fopen(filename, "w+");
	if(fp == NULL) {
		// TODO: better error handling in write
		return;
	}

	bundle_write(fp, b);
	fclose(fp);
}

static void skip_space(FILE *fp)
{
	int c;

	while(isspace(c = getc(fp)) && !feof(fp));
	if(!isspace(c)) {
		ungetc(c, fp);
	}
}

bundle_t *bundle_read(FILE *fp)
{
	int dim, count, length, i, j, k, n;
	double f;
	bundle_t *b;
	path_t *path;
	point_t p;
	bool good_path;

	skip_space(fp);
	n = fscanf(fp, "dim = %d\n", &dim);
	if(n != 1) {
		return NULL;
	}

	skip_space(fp);
	n = fscanf(fp, "count = %d\n", &count);
	if(n != 1) {
		return NULL;
	}

	b = bundle_create(dim, count);

	for(i = 0; i < count; i++) {
		length = 0;
		skip_space(fp);
		n = fscanf(fp, "length = %d\n", &length);
		if(n != 1) {
			break;
		}

		path = path_create(dim, length);

		for(j = 0; j < length; j++) {
			p = path_get(path, j);

			for(k = 0; k < dim; k++) {
				skip_space(fp);
				n = fscanf(fp, "%lg", &f);
				if(n != 1) {
					path_destroy(path);
					return b;
				}
				
				p[k] = (coord_t)f;
			}
		}

		bundle_set(b, i, path);
	}

	return b;
}

bundle_t *bundle_read_file_name(const char *filename)
{
	FILE *fp;
	bundle_t *b;

	fp = fopen(filename, "r");
	if(fp == NULL) {
		return NULL;
	}

	b = bundle_read(fp);

	fclose(fp);

	return b;
}
