#include "image/io.h"
#include "edge_tensors.h"

#include "line_graph.h"
#include "weights.h"
#include "bent_weight.h"
#include "rect_endpoints.h"

#include "optimize_glpk.h"
#include "min_spline_weight.h"

#include "draw_png.h"

int main(int argc, char **argv)
{
	image_float_t *I = NULL;
  tensor_field_t *tf = NULL;
	line_graph_t *G = NULL, *H = NULL;
	int *endpoints = NULL;

  draw_options_t draw_opts;

  assert(argc > 2);

	// Load image edgels
	I = image_float_load(argv[1]);
	tf = edge_tensors(I, 1, 1e-1, 3, &endpoints);

	//printf("%d edgels\n", tensor_field_count(tf));

  endpoints = rect_endpoints(tf);
	// Contour inference
	bent_weight_options_t bent_opts = {4, 1e1, 2*1024};
	weight_options_t weight_opts = {1, &bent_weight_expectation, &bent_opts};

	bent_opts.angle_weight = 1e2;

	G = weighted_line_graph(&weight_opts, tf);

	optimize_options_t opt_opts;
	opt_opts.lambda = 2e-3;
	opt_opts.mu = 1e-2;
	opt_opts.endpoints = endpoints;
	opt_opts.solver = SOLVER_DEFAULT;

	printf("Finding optimal contours...\n");
  H = optimal_contours(&opt_opts, G);

	// Draw
  draw_opts = draw_default_options;
  draw_opts.tensor_scaling = 4e-1;

  draw_png(argv[2], 480, 480, &draw_opts, tf, H);

	// Cleanup
  line_graph_destroy(H);
	line_graph_destroy(G);
	free(endpoints);
	tensor_field_destroy(tf);
	image_float_destroy(I);

	return 0;
}
