#include "line_graph_io.h"
#include "mnemosyne/mnemosyne.h"

void line_graph_write(mnemosyne_t *mn, line_graph_t *L)
{
  int i, j, n, m;
  edge_pair_t *e;

  n = line_graph_vertex_count(L);

	// Write the size of the line graph
	mnemosyne_write_uint32(mn, (uint32_t)n);

	// Iterate
  for(i = 0; i < n; i++) {
    m = line_graph_edge_pair_count(L, i);
    for(j = 0; j < m; j++) {
      e = line_graph_edge_pair(L, i, j);

			// Write this edge pair
			mnemosyne_write_uint32(mn, (uint32_t)(e->neighbors[0]));
			mnemosyne_write_uint32(mn, (uint32_t)(e->common_vertex));
			mnemosyne_write_uint32(mn, (uint32_t)(e->neighbors[1]));
			mnemosyne_write_double(mn, (uint32_t)(e->weight));
    }
  }
}

line_graph_t *line_graph_read(mnemosyne_t *mn)
{
	line_graph_t *L;
	int n;
	edge_pair_t ep;

	// Read the vertex count of the line graph
	n = (int)mnemosyne_read_uint32(mn);
	L = line_graph_create(n);

	// Read edge pairs
	while(true) {
		ep.neighbors[0] = (int)mnemosyne_read_uint32(mn);
		ep.common_vertex = (int)mnemosyne_read_uint32(mn);
		ep.neighbors[1] = (int)mnemosyne_read_uint32(mn);
		ep.weight = mnemosyne_read_double(mn);

		if(mnemosyne_eof(mn)) {
			break;
		}
		
    line_graph_add_edge_pair(L, &ep);
	}

	return L;
}

line_graph_t *line_graph_read_file(FILE *fp)
{
  line_graph_t *G;
  int n, s;
  edge_pair_t e;
  double w;

  // Read in size
  s = fscanf(fp, "n = %d", &n);
	if(s != 1) {
		return NULL;
	}
  G = line_graph_create(n);
  
  // Read in edge pairs
  while(1) {
    s = fscanf(fp, "%d %d %d %le",
               &e.neighbors[0],
               &e.common_vertex,
               &e.neighbors[1],
               &w);
               
    e.weight = w;

    if(s < 4 || feof(fp)) {
      break;
    }

    line_graph_add_edge_pair(G, &e);
  }
  

  return G;
}

line_graph_t *line_graph_read_file_name(const char *path)
{
  FILE *fp;
  line_graph_t *G;
  
  fp = fopen(path, "r");
  if(fp == NULL) {
    return NULL;
  }

  G = line_graph_read_file(fp);

  fclose(fp);

  return G;
}

int line_graph_write_file(FILE *fp, line_graph_t *G)
{
  int i, j, n, m, s;
  edge_pair_t *e;

  n = line_graph_vertex_count(G);
  fprintf(fp, "n = %d\n", n);

  for(i = 0; i < n; i++) {
    m = line_graph_edge_pair_count(G, i);
    for(j = 0; j < m; j++) {
      e = line_graph_edge_pair(G, i, j);

      s = fprintf(fp, "%d %d %d %.10le\n",
                  e->neighbors[0],
                  e->common_vertex,
                  e->neighbors[1],
                  e->weight);

      if(s < 0) {
        return s;
      }
    }
  }

  return 0;
}

int line_graph_write_file_name(const char *path, line_graph_t *G)
{
  FILE *fp;
  int s;
  
  fp = fopen(path, "w+");
  if(fp == NULL) {
    return errno;
  }

  s = line_graph_write_file(fp, G);
  if(s) {
    return s;
  }

  return fclose(fp);
}
