#include "mask_io.h"
#include "nifti.h"

#include "cli.h"

static int argc;
static char **argv;

static bool verbose = false;
static mask_t *subset = 0, *mask = 0;
static char out_path[PATH_MAX + 1];

static int set_verbose(void *closure_data, const argdef_t *def, const char *value);


static int set_subset(void *closure_data, const paramdef_t *def, const char *filename);
static int set_in(void *closure_data, const paramdef_t *def, const char *filename);
static int set_out(void *closure_data, const paramdef_t *def, const char *filename);

// Set up CLI
static const argdef_t arguments[] = {
	// General options
  {'v', "verbose", 0,
   NULL, ARG_VALUE_OPTIONAL, NULL,
   "Whether to print status information during weight calculation and optimization.",
	 &set_verbose},
	
	{0}
};

static const arggroup_t groups[] = {
	{0}
};

static const paramdef_t params[] = {
  {"subset", "The subset for which we want the transformed mask.", ARG_REQUIRED,
	 &set_subset},

  {"in", "Input mask.", ARG_REQUIRED,
	 &set_in},
	
	{"out", "Output mask.", ARG_REQUIRED,
	 &set_out},

  {0}
};

static const cli_t cli = {
  arguments,
  groups,
  params,
  "mask-subset",
  "Transform a mask into the indices of a subset.",
  "Curvature-LP",
  "0.1",
  "Maxwell Collins"
};

int main(int _argc, char **_argv)
{
	mask_t *out;

  // Handle arguments
	argc = _argc;
	argv = _argv;

	cli_argparse(&cli, argc, argv, NULL);

	// Perform mapping
	out = mask_subset_indices(subset, mask);

	// Write mask
	mask_write_file_name(out_path, out);

	return 0;
}

static int set_verbose(void *closure_data, const argdef_t *def, const char *value)
{
	verbose = true;
	return 0;
}

static mask_t *load_mask(const char *filename)
{
	mask_t *mask;

	if(nifti_filename(filename)) {
		mask = nifti_load_mask(filename, &nifti_load_options_default, NULL);
	}
	else {
		mask = mask_from_file_name(filename);
	}

	if(mask == NULL) {
		fprintf(stderr, "Error reading input mask file \"%s\"\n", filename);
		exit(EX_NOINPUT);
  }

	return mask;
}

static int set_subset(void *closure_data, const paramdef_t *def, const char *filename)
{
	subset = load_mask(filename);

	return 0;
}

static int set_in(void *closure_data, const paramdef_t *def, const char *filename)
{
	mask = load_mask(filename);

	return 0;
}

static int set_out(void *closure_data, const paramdef_t *def, const char *filename)
{
	strncpy(out_path, filename, PATH_MAX);
	out_path[PATH_MAX] = '\0';

	return 0;
}
