#ifndef __TENSOR_CANVAS_HH__
#define __TENSOR_CANVAS_HH__

#include <GL/gl.h>
#include <GL/glu.h>

#include "model/model_canvas.hh"

extern "C" {
#include "tensor_field.h"
#include "line_graph.h"
#include "bundle.h"
}

class TensorCanvas: public ModelCanvas
{
public:
  TensorCanvas(
    wxWindow *parent,
    const wxPoint &pos = wxDefaultPosition, const wxSize &size = wxDefaultSize,
    const wxString &name = _("TensorCanvas"),
    tensor_field_t *init_tf = NULL, line_graph_t *G = NULL,
    bool draw_tensor_glyphs = true,
    bundle_t *paths = NULL,
    float tensor_scale = -1);
  virtual ~TensorCanvas();

  // tensor field property
  void setTensorField(tensor_field_t *tf);
  tensor_field_t *getTensorField();

  // line graph property
  void setLineGraph(line_graph_t *G);
  line_graph_t *getLineGraph();

  void setSpherePrecision(int);
  int getSpherePrecision();

  virtual void draw();

  DECLARE_EVENT_TABLE()

private:
  tensor_field_t *tf;
  line_graph_t *G;
  bundle_t *paths;

  coord_t min[3], max[3];
  coord_t max_lambda;
  float tensor_scale;

  GLUquadric *quad;
  int sphere_precision;

  bool draw_tensor_glyphs;
  bool use_display_lists, sphere_list_valid, field_list_valid;
  GLuint lists;

  void initGlyphLists();

  inline GLuint circleList() {return lists;}
  void drawCircle();
  inline GLuint sphereList() {return lists + 1;}
  void drawSphere();

  inline GLuint fieldList() {return lists + 2;}

  void drawTensor(point_t p, tensor_t *T);
  
  void fieldSpace();

  void drawField();
  void drawGraph();
  void drawPaths();
};

#endif // #ifndef __TENSOR_CANVAS_HH__
