#ifndef __MODEL_CANVAS_HH__
#define __MODEL_CANVAS_HH__

#include <wx/wx.h>
#include <wx/glcanvas.h>

#if !(wxUSE_GLCANVAS)
#error wxGLCanvas not supported 
#endif

class ModelCanvas: public wxGLCanvas
{
public:
  ModelCanvas(
    wxWindow *parent,
    const wxPoint &pos = wxDefaultPosition, const wxSize &size = wxDefaultSize,
    const wxString &name = _("ModelCanvas"),
    int dimensionality = 3,
    bool take_focus = true,
    bool left_button_rotates = false, bool right_button_rotates = true);
  virtual ~ModelCanvas();

  virtual void draw();

  void OnPaint(wxPaintEvent &event);
  void OnSize(wxSizeEvent &event);
  void OnKeyDown(wxKeyEvent& event);
  void OnEraseBackground(wxEraseEvent &event);

  void OnMouseLeftDown(wxMouseEvent &event);
  void OnMouseLeftUp(wxMouseEvent &event);
  void OnMouseRightDown(wxMouseEvent &event);
  void OnMouseRightUp(wxMouseEvent &event);
  void OnMouseMove(wxMouseEvent &event);

  void OnMouseWheel(wxMouseEvent &event);

  DECLARE_EVENT_TABLE()

protected:
  void SetDimensionality(int dim);
  int GetDimensionality();

private:
  void rotate(double increment, const double *axis);
  void zoom(double factor);
  void translate(double x, double y, double z);

  bool left_button_rotates, right_button_rotates;

  double last_x, last_y;
  bool left_dragging, right_dragging;

  double zoom_translation;

  int dimensionality;
};

#endif // #ifndef __MODEL_CANVAS_HH__
