#include "tensor_sim.h"

static void zero(tensor_field_t *tf)
{
	int i, n;

	n = tensor_field_count(tf);
	for(i = 0; i < n; i++) {
		tensor_zero(tensor_field_tensor(tf, i));
	}
}

static void add_voxel_segment(point_t p, point_t q, tensor_t *T)
{
	int dim;
	tensor_t *D;

	dim = tensor_dim(T);
	coord_t diff[dim];

	// Create the dyadic tensor
	char Ddata[tensor_size(dim)];
	D = tensor_create_at(dim, Ddata);

	point_sub(dim, diff, q, p);
	tensor_dyadic(D, diff);

	// Add into T
	tensor_add(T, D);
}

static void add_segment(point_t p, point_t q, tensor_field_t *tf)
{
	int dim;

	dim = tensor_field_dim(tf);

	// TODO
	// Split into segments based on voxel boundaries
	// Can copy code from gibbs tracking
}

static void add_path(path_t *path, tensor_field_t *tf)
{
	point_t p, q;
	int i, length;

	assert(path_dim(path) == tensor_field_dim(tf));

	length = path_length(path);
	for(i = 0; i < length - 1; i++) {
		p = path_get(path, i);
		q = path_get(path, i+1);

		add_segment(p, q, tf);
	}
}

static void normalize(tensor_field_t *tf)
{
	// TODO
	// May want to scale all-at-once
	// Otherwise scale each by \sum_i \lambda_i
}

static void noise_tensor(tensor_t *T, double mag)
{
	// TODO
	// Set T to be a noise tensor of the given magnitude
}

static bool foreground_tensor(tensor_t *T)
{
	// TODO

	return false;
}

static void noise(tensor_field_t *tf, double fg_noise, double bg_noise)
{
	int i, n, dim;
	tensor_t *N, *T;

	dim = tensor_field_dim(tf);

	// Allocate noise tensor
	char Ndata[tensor_size(dim)];
	N = tensor_create_at(dim, Ndata);

	// Add noise to each tensor
	n = tensor_field_count(tf);
	for(i = 0; i < n; i++) {
		T = tensor_field_tensor(tf, i);

		if(foreground_tensor(T)) {
			// Add foreground noise to foreground tensors
			noise_tensor(N, fg_noise);
			tensor_add(T, N);
		}
		else {
			// Replace background tensors with background noise
			noise_tensor(N, bg_noise);
			tensor_assign(T, N);
		}
	}
}

tensor_field_t *tensor_sim(bundle_t *paths, int *size,
													 double fg_noise, double bg_noise)
{
	tensor_field_t *tf;
	path_t *path;
	int i, dim, count;

	dim = bundle_dim(paths);
	
	coord_t min[dim], max[dim], spacing[dim];

	// Create the initial tensor grid
	bundle_extent(paths, min, max);
	for(i = 0; i < dim; i++) {
		spacing[i] = (max[i] - min[i]) / (size[i] - 1);
	}

	tf = tensor_field_create_grid(dim, min, spacing, size);
	zero(tf);

	// Add in the paths
	count = bundle_count(paths);
	for(i = 0; i < count; i++) {
		path = bundle_get(paths, i);
		add_path(path, tf);
	}

	// Finish it up
	normalize(tf);
	noise(tf, fg_noise, bg_noise);

	return tf;
}
