#include "catmull_rom.h"

spline_t *catmull_rom_spline(path_t *path, double tau)
{
  point_t p[3], m;
  int i, j, length, dim;
  spline_t *spline;

  dim = path_dim(path);
  length = path_length(path);
  if(length <= 1) {
    return NULL;
  }

  spline = spline_create(dim, length);

  if(length == 2) {
    // Special case of a straight line
    point_sub(dim, spline_m(spline, 0), path_get(path, 1), path_get(path, 0));
    point_assign(dim, spline_m(spline, 1), spline_m(spline, 0));

    for(i = 0; i < 2; i++) {
      point_assign(dim, spline_p(spline, i), path_get(path, i));
    }
  }
  else {
    // Normal Catmull-Rom spline
    for(i = 0; i < length; i++) {
      if(i == 0) {
        // Forward difference at first endpoint
        p[0] = path_get(path, 0);
        p[1] = path_get(path, 0);
        p[2] = path_get(path, 1);
      }
      else if(i == length - 1) {
        // Backward difference at last endpoint
        p[0] = path_get(path, length - 2);
        p[1] = path_get(path, length - 1);
        p[2] = path_get(path, length - 1);
      }
      else {
        for(j = 0; j < 3; j++) {
          p[j] = path_get(path, i + j - 1);
        }
      }

      point_assign(dim, spline_p(spline, i), p[1]);

      m = spline_m(spline, i);
      point_sub(dim, m, p[2], p[0]);
      point_mul(dim, m, tau);
    }
  }

  return spline;
}
