#include "min_combined.h"
#include "util.h"


spline_t *spline_test_create(int length, const spline_knot_t *knots)
{
  spline_t *s;
  int i;

  s = spline_create(length);
  for(i = 0; i < length; i++) {
    s->knots[i] = knots[i];
  }

  return s;
}

// Specify knot test cases
const spline_knot_t knots0[] = {
  // p          s
  {{-0.5, 0.0}, {0.0, 0.5}},
  {{0.0, 0.0}, {0.0, -0.5}},
  {{0.5, 0.0}, {0.0, 0.5}}
};

// Specify test cases
#define LENGTH(knot_list) (sizeof((knot_list)) / sizeof(spline_knot_t))

// Specify options
minimize_combined_options_t minimize_combined_options = {
  // minimize_options
  {
    1024, // max_iter
    1e-1, // tol
		{4*1024,}, // start_precision
    1 // verbose
	},
	4*1024, // max_integration_intervals

  2, // TC_power
  2, // arc_length_power

  1 // curvature_factor
};


static void print_magnitudes(spline_t *s)
{
	int i, n;
	double mag;

	n = s->length;
	for(i = 0; i < n; i++) {
		mag = point_l2_norm(spline_m(s, i));
		printf("%2.4e, ", mag);
	}
	printf("\n");
}

int main(int argc, char **argv)
{
	spline_t *s;
	int i, j, n, m;
	double mag;

	srand(time(NULL));

	for(i = 0; i < 10; i++) {
		m = LENGTH(knots0);
		s = spline_test_create(m, knots0);

		// Randomize magnitudes
		for(j = 0; j < m; j++) {
			mag = pow(10, frand(-2, 4));
			point_mul(spline_m(s, j), mag);
		}

		printf("Before: ");
		print_magnitudes(s);

		spline_magnitudes_min_combined(s, &minimize_combined_options);

		// Print magnitudes
		printf("After:  ");
		print_magnitudes(s);
		printf("\n");

		spline_destroy(s);
	}

	return 0;
}
