#ifndef __CROSS_PRODUCT_H__
#define __CROSS_PRODUCT_H__

#include "point.h"

// Define cross product
typedef struct {
  coord_t sign;
  int i, j;
} crossprod_def_term_t;

typedef struct {
  int term_count;
  const crossprod_def_term_t terms[];

} crossprod_def_t;

static const crossprod_def_t crossprod_def_2d = {2,
  {{ 1, 0, 1},
   {-1, 1, 0}}};

static const crossprod_def_t crossprod_def_3d = {6,
  {{1,  1, 2},
   {1,  2, 0},
   {1,  0, 1},
   {-1, 2, 1},
   {-1, 0, 2},
   {-1, 1, 0}}};

static inline const crossprod_def_t *crossprod_def(int dim)
{
  switch(dim) {
  case 2:
    return &crossprod_def_2d;
  case 3:
    return &crossprod_def_3d;

  default:
    fprintf(stderr, "Unsupported dimensionality (%s:%d)\n", __FILE__, __LINE__);
    exit(1);
  }
}

void crossprod3d(point_t dst, point_t u, point_t v);
length_t crossprod3d_norm(point_t u, point_t v);

/*
#if DIM == 2

#define CROSS_TERMS (2)

static const double cross_sign[2] = {1, -1};
static const int cross_i[2] =       {0, 1};
static const int cross_j[2] =       {1, 0};

#elif DIM == 3
#define CROSS_TERMS (6)

static const double cross_sign[6] = {1, 1, 1, -1, -1, -1};
static const int cross_i[6] =       {1, 2, 0,  2,  0,  1};
static const int cross_j[6] =       {2, 0, 1,  1,  2,  0};

#else
#error Unsupported dimensionality
#endif
*/


#endif // #ifndef __CROSS_PRODUCT__
