#include <stdio.h>

#include "current_spline.h"

#include "kd_tree.h"

#define SPLINE_POINTS (256)

static spline_t *_current_spline = NULL;

int spline_changed = 0;

static int dim;
static coord_t *spline_path = NULL;

static spatial_tree_t *spline_path_points = NULL;
static spatial_tree_options_t tree_opts = {8, 2};
static int *indices = NULL;;

int closest_point = -1;

spline_t *current_spline()
{
  spline_t *s;

  if(_current_spline == NULL) {
    s = _current_spline = spline_create(2, 3);

    // knot 0
    spline_p(s, 0)[0] = 0.5;
    spline_p(s, 0)[1] = 0.0;

    spline_m(s, 0)[0] = 0.0;
    spline_m(s, 0)[1] = 0.5;

    // knot 1
    spline_p(s, 1)[0] = 0.0;
    spline_p(s, 1)[1] = 0.0;

    spline_m(s, 1)[0] = 0.0;
    spline_m(s, 1)[1] = -0.5;

    // knot 2
    spline_p(s, 2)[0] = -0.5;
    spline_p(s, 2)[1] = 0.0;

    spline_m(s, 2)[0] = 0.0;
    spline_m(s, 2)[1] = 0.5;
  }

  return _current_spline;
}

int current_spline_path_count()
{
  return SPLINE_POINTS * (_current_spline->length - 1);
}

int current_spline_dim()
{
  return current_spline()->dim;
}

param_t current_spline_t(int i)
{
  return (param_t)i/(current_spline_path_count()-1)
    * (_current_spline->length - 1);
}

point_t current_spline_point(int i)
{
  if(spline_path == NULL) {
    return NULL;
  }

  return spline_path + i * dim;
}

int current_spline_closest(point_t x)
{
  spatial_tree_entry_t *closest;

  if(spline_path_points == NULL) {
    return -1;
  }

  // Find closest point
  spatial_tree_nearest_neighbor(
    spline_path_points, x,
    1, &closest);

  return *(int *)closest->data;
}

void current_spline_refresh()
{
  unsigned int i, order, N;
  size_t s;

  dim = current_spline()->dim;
  N = current_spline_path_count();

  // Calculate waypoints of spline
  s = sizeof(coord_t) * dim * N;
  spline_path = (coord_t *)realloc(spline_path, s);
  spline_calculate(current_spline(), N, spline_path);

  // Refresh tree
  if(spline_path_points == NULL) {
    s = sizeof(int) * N;
    indices = (int *)realloc(indices, s);
    for(i = 0; i < N; i++) {
      indices[i] = i;
    }
  }
  else {
    spatial_tree_destroy(spline_path_points);
  }
  spline_path_points = spatial_tree_create(&tree_opts);

  // Add points in order of repeated bisection
  for(order = N; order != 0; order /= 2) {
    for(i = order; i < current_spline_path_count(); i += order) {
      if((i % (2*order)) != 0) {
        spatial_tree_add(spline_path_points, spline_path + dim * i, indices + i);
      }
    }
  }
}


curvature_t current_spline_TC()
{
	return spline_total_curvature(current_spline(), 1e-6, 8192);
}

static void change_dim(int dst_dim)
{
  spline_t *s;
	int i, n;
	int src_dim, d;

	// Figure out dimensions to copy
	src_dim = current_spline_dim();
	d = src_dim < dst_dim ? src_dim : dst_dim;

	n = _current_spline->length;
	
	// Create the new spline
	s = spline_create(dst_dim, n);

	// Copy over points
	for(i = 0; i < n; i++) {
		point_assign(d, spline_p(s, i), spline_p(_current_spline, i));
		point_assign(d, spline_m(s, i), spline_m(_current_spline, i));
	}

	// Make the new spline the current spline
	spline_destroy(_current_spline);
	_current_spline = s;

	current_spline_refresh();
}

void current_spline_to_2d()
{
	change_dim(2);
}

void current_spline_to_3d()
{
	change_dim(3);
}

void current_spline_to_toggle_dim()
{
	if(current_spline_dim() == 2) {
		change_dim(3);
	}
	else {
		change_dim(2);
	}
}
