#include <GL/gl.h>
#include <SDL.h>
#include <SDL_ttf.h>

#include "draw.h"
#include "point.h"
#include "spline.h"
#include "current_spline.h"


static void draw_arrow(int dim, point_t p, point_t v)
{
  coord_t u[dim], end[dim];
  double wing_length;

  unit_vector(dim, v, u);
  point_add(dim, end, p, v);

  wing_length = 3e-2;

  glBegin(GL_LINES);

  gl_point(dim, p);
  gl_point(dim, end);

  /*
		TODO

  glVertex2d(x + vx, y + vy);
  glVertex2d(x + vx - u[0]*wing_length + u[1]*wing_length,
             y + vy - u[1]*wing_length - u[0]*wing_length);

  glVertex2d(x + vx, y + vy);
  glVertex2d(x + vx - u[0]*wing_length - u[1]*wing_length,
             y + vy - u[1]*wing_length + u[0]*wing_length);
  */

  glEnd();
}

static void draw_circle(int dim, point_t center, point_t axis, float r, int N)
{
  double theta;
  int i;
	coord_t p[dim], u[dim];

	for(i = 0; i < dim; i++) {
		p[i] = 0;
	}

	glPushMatrix();
	gl_translate(dim, center);
	glScalef(r, r, r);
	
	if(dim == 2) {
	}
	else if(dim == 3) {
		// TODO: rotate axis to z
		// rotate around axis of cross product with z-axis
		// by angle between them
	}
	else {
		UNSUPPORTED_DIMENSIONALITY(dim);
	}

  glBegin(GL_LINE_STRIP);
  for(i = 0; i < N; i++) {
    theta = (2 * M_PI) * (double)i / (N-1);
		
		p[0] = cos(theta);
		p[1] = sin(theta);
		
		gl_point(dim, p);
  }
  glEnd();

	glPopMatrix();
}



static void draw_point_statistics(int i)
{
  spline_t *s;
	int j, dim;
  param_t t;
	curvature_t kappa, R;
  point_t p;

  s = current_spline();
	dim = s->dim;

	coord_t dp[dim], dpu[dim], ddp[dim], ddpu[dim];
	coord_t circle_center[dim], circle_axis[dim];

  // Find tangent and curvature
  t = current_spline_t(i);

  spline_derivative(s, t, dp);
  unit_vector(dim, dp, dpu);

	spline_derivative2(s, t, ddp);
	unit_vector(dim, ddp, ddpu);

	// TODO: signed curvature...
  kappa = spline_curvature(s, t);
  R = 1/kappa;

  p = current_spline_point(i);
  
	// Find center of osculating circle
	if(dim == 2) {
		circle_center[0] = p[0] - dpu[1] * R;
		circle_center[1] = p[1] + dpu[0] * R;
	}
	// else if(dim == 3) {
	// TODO
	// }
	else {
		UNSUPPORTED_DIMENSIONALITY(dim);
	}

  // Draw tangent and osculating circle
  glColor3f(1.0, 0.0, 0.0);
	draw_circle(dim, circle_center, NULL, fabs(R), 256);
  //draw_arrow(p[0], p[1], dp[0]/4, dp[1]/4);
  //draw_circle(p[0] - dpu[1]*R, p[1] + dpu[0]*R, fabs(R), 256);
}


void draw()
{
  glColor3f(1.0, 1.0, 1.0);
  draw_spline();

  glColor3f(0.0, 0.0, 1.0);
  draw_spline_markers();

  if(closest_point != -1) {
    draw_point_statistics(closest_point);
  }
}
