#include "input.h"
#include "draw.h"
#include "point.h"
#include "spline.h"
#include "current_spline.h"

#include "util.h"

#include "min_TC.h"
#include "min_arc_length.h"
#include "min_combined.h"


#define POSITION (0)
#define DIRECTION (1)

static int captured_point_type = -1;
static int captured_point = -1;

static void world_coords_button(point_t m, SDL_MouseButtonEvent *e)
{
  m[0] = (double)(2 * (e->x - width/2))/width;
  m[1] = (double)(-2 * (e->y - height/2))/height;
}

static void world_coords_motion(point_t m, SDL_MouseMotionEvent *e)
{
  m[0] = (double)(2 * (e->x - width/2))/width;
  m[1] = (double)(-2 * (e->y - height/2))/height;
}

void mouseButton(SDL_MouseButtonEvent *e)
{
  int i = 0, dim;
  point_t p, m;
  spline_t *s;

  s = current_spline();
  dim = s->dim;

  coord_t mouse[dim], v[dim];

  switch(e->type) {
  case SDL_MOUSEBUTTONDOWN:

    switch(e->button) {
    case SDL_BUTTON_LEFT:
      world_coords_button(mouse, e);

      for(i = 0; i < s->length; i++) {
        // Find marker points
        p = spline_p(s, i);
        m = spline_m(s, i);

        point_assign(dim, v, m);
        point_mul(dim, v, 0.25);
        point_add(dim, v, p, v);

        // Check distance
        if(point_l1_dist(dim, mouse, v) <= MARKER_SIZE) {
          captured_point_type = DIRECTION;
          captured_point = i;
          break;
        }
        else if(point_l1_dist(dim, mouse, p) <= MARKER_SIZE) {
          captured_point_type = POSITION;
          captured_point = i;
          break;
        }
      }

      break;
    }
    break;

  case SDL_MOUSEBUTTONUP:
    switch(e->button) {
    case SDL_BUTTON_LEFT:
      captured_point_type = -1;
      captured_point = -1;
    }
    break;
    
  }
}

void mouseMotion(SDL_MouseMotionEvent *e)
{
  int i, dim;

  Uint8 *keys;
  
  double mag;

  spline_t *s;

  dim = current_spline_dim();
  coord_t mouse[dim];
  world_coords_motion(mouse, e);

  if(captured_point == -1) {
    // Find closest point
    i = current_spline_closest(mouse);

    if(i != -1 && point_l2_dist(dim, mouse, current_spline_point(i)) < 1e-1) {
      closest_point = i;
    }
    else {
      closest_point = -1;
    }
  }
  else {
    s = current_spline();

    if(captured_point_type == DIRECTION) {
      // Determine whether we want to just pay attention to magnitude
      keys = SDL_GetKeyState(NULL);

      // Convert to vector
      point_sub(dim, mouse, mouse, spline_p(s, captured_point));
      if(keys[SDLK_LCTRL] || keys[SDLK_RCTRL]) {
        point_projection(dim, mouse, spline_m(s, captured_point), mouse);
      }
      point_mul(dim, mouse, 4);

      // Assign
      if(point_l2_norm(dim, mouse) > 1e-4) {
        point_assign(dim, spline_m(s, captured_point), mouse);
      }
    }
    else {
      assert(captured_point_type == POSITION);

      point_assign(dim, spline_p(s, captured_point), mouse);
    }

    current_spline_refresh();
    spline_changed = 1;
  }
}

void keyEvent(SDL_KeyboardEvent *e)
{
  double sigma, c;
	minimize_combined_options_t min_opts;

  if(e->type == SDL_KEYUP) {
    switch(e->keysym.sym) {

    case SDLK_m:
			min_opts = minimize_combined_options_default;

      min_opts.minimization_options.max_iter = 32;
      min_opts.minimization_options.tol = 1e-1;
      min_opts.minimization_options.start_precision.N = 64;
      
      //sigma = 4;
      //c = (-16 * log(0.1) * (sigma - 1)) / square(M_PI);
      //min_opts.curvature_factor = c;

      //printf("c = %2f\n", c);

      printf("Picking derivative constraint magnitudes...\n");
      spline_magnitudes_min_combined(current_spline(), &min_opts);
      current_spline_refresh();

    case SDLK_c:
      printf("Spline total curvature: %2f\n", current_spline_TC());

      spline_changed = 0;
      break;

		case SDLK_d:
			current_spline_to_toggle_dim();
			spline_changed = 1;
			break;

    }
  }
}
