#include <SDL.h>
#include <SDL_ttf.h>

#include "draw.h"
#include "spline.h"
#include "current_spline.h"
#include "input.h"
#include "time.h"

int main(int argc, char **argv)
{
  int s;
  SDL_Surface *screen;
  SDL_Event event;

  srand(time(NULL));

  printf("Spline total curvature: %2f\n", current_spline_TC());
  current_spline_refresh();

  if(argc > 1 && strcmp(argv[1], "-nodisplay") == 0) {
    return 0;
  }

  // Start up SDL
  s = SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
  if(s != 0) {
    fprintf(stderr, "Could not initialize SDL: %s\n", SDL_GetError());
    return 1;
  }

  atexit(SDL_Quit);

  // Start up SDL_ttf
  s = TTF_Init();
  if(s != 0) {
    fprintf(stderr, "Could not initialize SDL_ttf: %s\n", TTF_GetError());
    return 1;
  }

  atexit(TTF_Quit);

  // Start up the screen
  s = SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
  if(s != 0) {
    fprintf(stderr, "Could not set up SDL: %s\n", SDL_GetError());
  }
  screen = SDL_SetVideoMode(width, height, 0, SDL_OPENGL | SDL_RESIZABLE);
  if(screen == NULL) {
    fprintf(stderr, "Could open OpenGL window: %s\n", SDL_GetError());
    exit(1);
  }

  SDL_WM_SetCaption("Spline World", "Spline World");

  // Main loop
  while(1) {

    // Handle events
    while(SDL_PollEvent(&event)) {
      switch(event.type) {

      case SDL_KEYDOWN:
      case SDL_KEYUP:
        keyEvent(&event.key);
        break;

      case SDL_MOUSEMOTION:
        mouseMotion(&event.motion);
        break;

      case SDL_MOUSEBUTTONDOWN:
      case SDL_MOUSEBUTTONUP:
        mouseButton(&event.button);
        break;

      case SDL_VIDEORESIZE:
        width = event.resize.w;
        height = event.resize.h;
        screen = SDL_SetVideoMode(width, height, 0, SDL_OPENGL | SDL_RESIZABLE);
        break;

      case SDL_QUIT:
        exit(0);

        //default:
        //	fprintf(stderr, "Unrecognized event %d\n", (int)event.type);

      }
    }

    // Update screen
    draw();
    SDL_GL_SwapBuffers();

    // Update calculations
    if(spline_changed) {
			printf("Spline total curvature: %2f\n", current_spline_TC());
      spline_changed = 0;
    }

    // Sleep for some length of time
    idle();
  }

  return 0;
}
