#include "min_TC.h"
#include "min_spline.h"

#include "TC.h"

static void print_error(minimize_options_t *opts, int err, const char *msg)
{
	if(opts->verbose) {
		fputs(msg, stderr);
		fputc('\n', stderr);
	}
}

const minimize_TC_options_t minimize_TC_options_default = {
  // minimize_options
  {
    32, // max_iter
    8e-1, // tol
		{128,}, // start_precision
    0, // verbose
		&print_error, // error
		NULL // user_data
	},
	32*1024, // max_integration_intervals,
  
  2 // TC_power
};

static double objective(void *closure_data, spline_t *spline, precision_t e)
{
	minimize_TC_options_t *opts;

	opts = (minimize_TC_options_t *)closure_data;

  return TC(spline, opts->TC_power, e.N);
}


static void gradient(void *closure_data, spline_t *s, int dim, double *g, precision_t e)
{
	minimize_TC_options_t *opts;
	int i;

	opts = (minimize_TC_options_t *)closure_data;

	assert(dim == s->length);

	// Grab gradient elementwise
	for(i = 0; i < dim; i++) {
    g[i] = TC_magnitude_gradient(s, i, opts->TC_power, e.N);
	}
}

static int refine(void *closure_data, precision_t *e)
{
	e->N *= 2;
	
	return e->N <= 64*1024;
}

double spline_magnitudes_min_TC(spline_t *s, minimize_TC_options_t *opts)
{
	return spline_optimize_magnitudes(s,
		&objective, &gradient, &refine,
		opts, &opts->minimization_options);
}
