#include "min_arc_length.h"
#include "min_spline.h"
#include "arc_length.h"

static void print_error(minimize_options_t *opts, int err, const char *msg)
{
	if(opts->verbose) {
		fputs(msg, stderr);
		fputc('\n', stderr);
	}
}

const minimize_arc_length_options_t minimize_arc_length_options_default = {
  // minimize_options
  {
    32, // max_iter
    8e-1, // tol
		{128,}, // start_precision
    0, // verbose
		&print_error, // error
		NULL // user_data
	},

	32*1024, // max_integration_intervals
  2 // arc_length_power
};


static double objective(void *closure_data, spline_t *spline, precision_t e)
{
  minimize_arc_length_options_t *opts;

  opts = (minimize_arc_length_options_t *)closure_data;

  return arc_length(spline, opts->arc_length_power, e.N);
}

static void gradient(void *closure_data, spline_t *s, int dim, double *g, precision_t e)
{
  minimize_arc_length_options_t *opts;
  int i;

  opts = (minimize_arc_length_options_t *)closure_data;

  assert(dim == s->length);
  
	// Grab gradient elementwise
	for(i = 0; i < dim; i++) {
    g[i] = arc_length_magnitude_gradient(s, i, opts->arc_length_power, e.N);
	}
}


static int refine(void *closure_data, precision_t *e)
{
	e->N *= 2;

	return e->N <= 64*1024;
}

double spline_magnitudes_min_arc_length(spline_t *s, minimize_arc_length_options_t *opts)
{
	return spline_optimize_magnitudes(s,
		&objective, &gradient, &refine,
		opts, &opts->minimization_options);
}
