#include "min_combined.h"

#include "arc_length.h"
#include "TC.h"

static void print_error(minimize_options_t *opts, int err, const char *msg)
{
	if(opts->verbose) {
		fputs(msg, stderr);
		fputc('\n', stderr);
	}
}

const minimize_combined_options_t minimize_combined_options_default = {
  // minimize_options
  {
    32, // max_iter
    8e-1, // tol
		{128,}, // start_precision
    0, // verbose
		&print_error, // error
		NULL // user_data
	},
	1024, // max_integration_intervals

  2, // TC_power
  2, // arc_length_power

  1 // curvature_factor
};

static double objective(void *closure_data, spline_t *spline, precision_t e)
{
  minimize_combined_options_t *opts;
  double s, kappa;

  opts = (minimize_combined_options_t *)closure_data;

  s = arc_length(spline, opts->arc_length_power, e.N);
  kappa = TC(spline, opts->TC_power, e.N);
	//spline_print(spline);

	//printf("s = %lf, kappa = %lf\n", s, kappa);

  return s + opts->curvature_factor * kappa;
}

double min_combined_objective(spline_t *s, minimize_combined_options_t *opts)
{
  precision_t e;

  e.N = opts->max_integration_intervals;

  return objective(opts, s, e);
}

static void gradient(void *closure_data, spline_t *spline, int dim, double *g, precision_t e)
{
  minimize_combined_options_t *opts;
	int i;
  double ds, dkappa;

  opts = (minimize_combined_options_t *)closure_data;

  assert(dim == spline->length);

  for(i = 0; i < dim; i++) {
    ds = arc_length_magnitude_gradient(spline, i, opts->arc_length_power, e.N);
    dkappa = TC_magnitude_gradient(spline, i, opts->TC_power, e.N);

    g[i] = ds + opts->curvature_factor * dkappa;
  }
}

static int refine(void *closure_data, precision_t *e)
{
  minimize_combined_options_t *opts;

  opts = (minimize_combined_options_t *)closure_data;
	e->N *= 2;

	return e->N <= opts->max_integration_intervals;
}

double spline_magnitudes_min_combined(
  spline_t *s,
  minimize_combined_options_t *opts)
{
	return spline_optimize_magnitudes(s,
		&objective, &gradient, &refine,
		opts, &opts->minimization_options);
}
