#include "paces.h"
#include "min_combined.h"


spline_t *spline_test_create(int length, const spline_knot_t *knots)
{
  spline_t *s;
  int i;

  s = spline_create(length);
  for(i = 0; i < length; i++) {
    s->knots[i] = knots[i];
  }

  return s;
}

// Specify knot test cases
const spline_knot_t knots0[] = {
  // p          s
  {{-0.5, 0.0}, {0.0, 0.5}},
  {{0.0, 0.0}, {0.0, -0.5}},
  {{0.5, 0.0}, {0.0, 0.5}}
};

const spline_knot_t knots1[] = {
  // p          s
  {{-0.5, 0.0}, {0.0, 0.5}},
  {{0.0, 0.5}, {-0.5, 0.0}},
  {{0.5, 0.0}, {0.0, 0.5}}
};

const spline_knot_t knots2[] = {
  // p          s
  {{-0.5, 0.0}, {1.0, 0.1}},
  {{0.0, 0.0},  {1.0, -0.1}},
  {{0.5, 0.0},  {1.0, 0.1}}
};

const spline_knot_t knots3[] = {
  // p          s
  {{-0.5, 0.0}, {1.0, -1e-2}},
  {{0.0, 0.0},  {1.0, 1e-2}},
  {{0.5, 0.0},  {1.0, -1e-2}}
};

const spline_knot_t knots4[] = {
  // p          s
  {{-0.5, 0.0}, {-1.0, -1.0}},
  {{0.0, 0.0},  {-1.0, 1.0}},
  {{0.5, 0.5},  {1.0, 0.0}}
};

const spline_knot_t knots5[] = {
  // p          s
  {{-0.5, 0.5}, {0.0, 1.0}},
  {{0.0, 0.0},  {0.0, 1.0}},
  {{0.5, 0.5},  {0.0, 1.0}}
};

// Specify test cases
#define LENGTH(knot_list) (sizeof((knot_list)) / sizeof(spline_knot_t))


typedef struct {
  unsigned int length;
  const spline_knot_t *knots;
} test_case_t;

#define TEST_CASE(knot_list) {LENGTH(knot_list), (knot_list)}

const test_case_t test_cases[] = {
  TEST_CASE(knots0),
  TEST_CASE(knots1),
  TEST_CASE(knots2),
  TEST_CASE(knots3),
  TEST_CASE(knots4),
  TEST_CASE(knots5)
};

#define TEST_CASE_COUNT (sizeof(test_cases) / sizeof(test_case_t))

// Specify options
minimize_combined_options_t minimize_combined_options = {
  // minimize_options
  {
    8, // max_iter
    1e-1, // tol
		{32,}, // start_precision
    1 // verbose
	},
	4*1024, // max_integration_intervals

  2, // TC_power
  2, // arc_length_power

  1 // curvature_factor
};

int main(int argc, char **argv)
{
  int i;
  spline_t *s;
  double obj[TEST_CASE_COUNT];

  for(i = 0; i < TEST_CASE_COUNT; i++) {
    s = spline_test_create(test_cases[i].length, test_cases[i].knots);

    printf("Test case %d\n", i);
    spline_magnitudes_min_combined(s, &minimize_combined_options);
    obj[i] = min_combined_objective(s, &minimize_combined_options);
    printf("obj = %2f\n\n", obj[i]);
    printf("------------------------------------------------\n");
      

    spline_destroy(s);
  }

	printf("Final objectives:\n");
  for(i = 0; i < TEST_CASE_COUNT; i++) {
		printf("%02d: %2f\n", i, obj[i]);
	}
	

  return 0;
}
