#ifndef __SPLINE_H__
#define __SPLINE_H__

#include "point.h"
#include "path.h"

typedef struct spline spline_t;
struct spline {
  unsigned int dim, length;

  coord_t *data;
};

spline_t *spline_create(unsigned int dim, unsigned int length);
spline_t *spline_copy(const spline_t *);
void spline_destroy(spline_t *spline);

static inline coord_t *spline_p(spline_t *s, int i)
{
  return s->data + s->dim * (2 * i);
}

static inline coord_t *spline_m(spline_t *s, int i)
{
  return s->data + s->dim * (2 * i + 1);
}

typedef struct spline_knot spline_knot_t;
struct spline_knot {
  point_t p;
  point_t m;
};

static inline spline_knot_t spline_knot(spline_t *s, int i)
{
  spline_knot_t k;

  k.p = spline_p(s, i);
  k.m = spline_m(s, i);

  return k;
}

void spline_anchor(spline_t *s, int i, point_t p, point_t m);

// Calculate intermediate points + properties
typedef coord_t param_t;
typedef length_t speed_t;
typedef coord_t curvature_t;

void spline_point(spline_t *s, param_t t, point_t p_t);
void spline_derivative(spline_t *s, param_t t, point_t dp_t);
void spline_derivative2(spline_t *s, param_t t, point_t ddp_t);
curvature_t spline_curvature(spline_t *s, param_t t);
speed_t spline_speed(spline_t *s, param_t t);

void spline_calculate(spline_t *s, int N, coord_t *path);
path_t *spline_path(spline_t *s, int N);

// Total properties
curvature_t spline_total_curvature(spline_t *s, double epsilon, int max_evals);
length_t spline_arc_length(spline_t *s, double epsilon, int max_evals);

// I/O
void spline_print(spline_t *s);

#endif // #ifndef __SPLINE_H__
