#include "spline_canvas.hh"
#include "model/gl_util.hh"

extern "C" {
#include "min_combined.h"
}


/*
static const int DEFAULT_SPLINE_DIM = 2;
static const int DEFAULT_SPLINE_LENGTH = 3;

static const coord_t DEFAULT_SPLINE_POINTS[2 * DEFAULT_SPLINE_DIM * DEFAULT_SPLINE_LENGTH] = {
  // p            m
   0.5, 0.0,    0.0,  0.5,
   0.0, 0.0,    0.0, -0.5,
  -0.5, 0.0,    0.0,  0.5
};

//*/

///*
static const int DEFAULT_SPLINE_DIM = 3;
static const int DEFAULT_SPLINE_LENGTH = 3;

static const coord_t DEFAULT_SPLINE_POINTS[2 * DEFAULT_SPLINE_DIM * DEFAULT_SPLINE_LENGTH] = {
  // p                 m
   0.5, 0.0,  0.5,    0.0,  0.5, 0.5,
   0.0, 0.0,  0.0,    0.0, -0.5, 0.0,
  -0.5, 0.0, -0.5,    0.0,  0.5, 0.0
};
//*/

static const spline_t DEFAULT_SPLINE = {
  DEFAULT_SPLINE_DIM, // dim
  DEFAULT_SPLINE_LENGTH, // length
  const_cast<coord_t *>(DEFAULT_SPLINE_POINTS)};

int SplineCanvas::spline_points_count = 4096;
float SplineCanvas::marker_size = 2e-2;

/////////////////////// SplineCanvas //////////////////////

BEGIN_EVENT_TABLE(SplineCanvas, ModelCanvas)
  EVT_KEY_DOWN(SplineCanvas::OnKeyDown)
END_EVENT_TABLE()

SplineCanvas::SplineCanvas(
  wxWindow *parent,
  const wxPoint &pos, const wxSize &size,
  const wxString &name,
  const spline_t *init_spline, bool init_editable) :

  ModelCanvas(parent, pos, size, name,
              init_spline == NULL ? DEFAULT_SPLINE.dim : init_spline->dim),
  editable(init_editable)
{
  // Initialize spline
  if(init_spline == NULL) {
    spline = spline_copy(&DEFAULT_SPLINE);
  }
  else {
    spline = spline_copy(init_spline);
  }

  calculate_spline_points();
}

SplineCanvas::~SplineCanvas()
{
  spline_destroy(spline);
}

///////////////////////// drawing //////////////////////


void SplineCanvas::draw()
{
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  
  glColor3f(1.0, 1.0, 1.0);
  draw_spline();
  
  glColor3f(0.0, 0.0, 1.0);
  draw_knots();

  glPopMatrix();
}

void SplineCanvas::draw_spline()
{
  int dim = spline->dim;

  glBegin(GL_LINE_STRIP);
  for(int i = 0; i < spline_points_count; i++) {
    gl_point(dim, spline_points + i * dim);
  }
  glEnd();
}


void SplineCanvas::calculate_spline_points()
{
  // Allocate space for spline points
  if(spline_points != NULL) {
    delete [] spline_points;
  }

  spline_points = new coord_t[spline_points_count * spline->dim];

  // Calculate
  spline_calculate(spline, spline_points_count, spline_points);
}

void SplineCanvas::draw_marker(int dim, point_t p)
{ 
  float s = marker_size;

  glPushMatrix();
  gl_translate(dim, p);

  if(dim == 2) {
    // Draw a diamond
    glBegin(GL_QUADS);
    glVertex2f( s,  0);
    glVertex2f( 0,  s);
    glVertex2f(-s,  0);
    glVertex2f( 0, -s);
    glEnd();
  }
  else if(dim == 3) {
    // Draw a bipyramid

    glBegin(GL_TRIANGLE_FAN);
    glVertex3f( s,  0,  0);
    glVertex3f( 0,  s,  0);
    glVertex3f( 0,  0,  s);
    glVertex3f( 0, -s,  0);
    glVertex3f( 0,  0, -s);
    glVertex3f( 0,  s,  0);
    glEnd();

    glBegin(GL_TRIANGLE_FAN);
    glVertex3f(-s,  0,  0);
    glVertex3f( 0,  s,  0);
    glVertex3f( 0,  0,  s);
    glVertex3f( 0, -s,  0);
    glVertex3f( 0,  0, -s);
    glVertex3f( 0,  s,  0);
    glEnd();

  }
  else {
    UNSUPPORTED_DIMENSIONALITY(dim);
  }

  glPopMatrix();
}

void SplineCanvas::draw_knots()
{
  int dim = spline->dim, n = spline->length;

  for(int i = 0; i < n; i++) {
    point_t p = spline_p(spline, i), m = spline_m(spline, i);
    coord_t m_scaled[dim], m_marker[dim];

    point_assign(dim, m_scaled, m);
    point_mul(dim, m_scaled, 0.25);
    point_add(dim, m_marker, p, m_scaled);

    draw_marker(dim, p);
    //draw_marker(dim, m_marker);
  }
}

///////////////////////// input //////////////////////

void SplineCanvas::OnKeyDown(wxKeyEvent& event)
{
  switch(event.GetKeyCode()) {
  case 'M': {
    cout<<"Picking derivative constraint magnitudes..."<<endl;

    minimize_combined_options_t min_opts = minimize_combined_options_default;

    min_opts.minimization_options.max_iter = 32;
    min_opts.minimization_options.tol = 1e-1;
    min_opts.minimization_options.start_precision.N = 4096;

    spline_magnitudes_min_combined(spline, &min_opts);

    calculate_spline_points();
    Refresh();
  } break;

  default:
    ModelCanvas::OnKeyDown(event);
  }
}
