#include "tract_file_struct.h"

void tract_file_init(
  tract_file_t *tf,
  const tract_file_vtable_t *vtable,
  FILE *fp)
{
	tf->vtable = vtable;

	//tf->status = STATUS_OK;
	// writable/seekable?

	tf->count = -1;
	tf->i = 0;

	tf->fp = fp;
}

int tract_file_close(tract_file_t *tf)
{
	int s;
	FILE *fp;

	s = tract_file_flush(tf);
	if(s != 0) {
		return s;
	}

	fp = tf->fp;

	if(tf->vtable->close == NULL) {
		free(tf);
	}
	else {
		s = (*tf->vtable->close)(tf);
		if(s != 0) {
			return s;
		}
	}

	return 0;
	/*
		TODO
	if(fclose(fp) == 0) {
		return STATUS_OK;
	}
	else {
		return STATUS_FILE_ERROR;
	}
	*/
}

/////////////////// Status /////////////////////

/*
int tract_file_writeable(tract_file_t *tf)
{
	return tf->writable;
}
*/

///////////////////// Read /////////////////////

int tract_file_count(tract_file_t *tf, int *n)
{
	path_t *tract;

	// Determine count
	if(tf->count == -1) {
		// TODO: seek back?

		while(!tract_file_end(tf)) {
			tract = tract_file_read(tf);
			if(tract == NULL) {
				return 1;
			}
			path_destroy(tract);
		}

		tract_file_tell(tf, &tf->count);
	}

	if(n != NULL) {
		*n = tf->count;
	}

	return 0;
}

int tract_file_tell(tract_file_t *tf, int *i)
{
	if(i != NULL) {
		*i = tf->i;
	}

	return 0;
}

path_t *tract_file_read(tract_file_t *tf)
{
	path_t *tract;

	tract = (*tf->vtable->read_tract)(tf);

	if(tract != NULL) {
		tf->i++;
	}

	return tract;
}


int tract_file_end(tract_file_t *tf)
{
	int c, eof;
	FILE *fp;

	if(tf->vtable->end == NULL) {
		fp = tf->fp;

		c = getc(fp);
		eof = feof(fp);
		ungetc(c, fp);

		return eof;
	}
	else {
		return (*tf->vtable->end)(tf);
	}
}

/////////////////// Editing ///////////////////

int tract_file_append(tract_file_t *tf, path_t *tract)
{
	int s;
	int n;

	int sought;
	off_t old_offset;

	// Seek to end of file
	if(!feof(tf->fp)) {
		sought = 1;
		old_offset = ftello(tf->fp);
		n = fseek(tf->fp, 0, SEEK_END);

		if(n != 0) {
			return 1;
		}
	}
	else {
		sought = 0;
		old_offset = 0;
	}

	// Write tract
	s = (*tf->vtable->write_tract)(tf, tract);
	if(s != 0) {
		return s;
	}

	if(tf->count != -1) {
		tf->count++;
	}

	// Restore old offset
	if(sought) {
		n = fseeko(tf->fp, old_offset, SEEK_SET);
		if(n != 0) {
			return 1;
		}
	}

	return 0;
}

int tract_file_flush(tract_file_t *tf)
{
	int s;
	int n;

	// Perform any file-specific flush actions
	if(tf->vtable->flush != NULL) {
		s = (*tf->vtable->flush)(tf);
		if(s != 0) {
			return s;
		}
	}

	// File-level flush
	n = fflush(tf->fp);
	if(n != 0) {
		return 1;
	}

	return 0;
}
