#ifndef __TRK_H__
#define __TRK_H__

#include "tract_file.h"

typedef struct trk_hdr trk_hdr_t;
typedef struct trk trk_t;


/**
 * The initial header to a trackvis .trk file.
 *
 * Specifies a coordinate system and sizing information on the tract data.
 *
 * @see http://www.trackvis.org/docs/?subsect=fileformat
 */
struct trk_hdr
{
 char id_string[6];
 int16_t dim[3];
 float voxel_size[3];
 float origin[3];
 int16_t n_scalars;
 char scalar_name[10][20];
 int16_t n_properties;
 char property_name[10][20];
 char reserved[508];
 char voxel_order[4];
 char pad2[4];
 float image_orientation_patient[6];
 char pad1[2];
 unsigned char invert_x;
 unsigned char invert_y;
 unsigned char invert_z;//July 24, 2009.
 unsigned char swap_xy;
 unsigned char swap_yz;
 unsigned char swap_zx;
 int32_t n_count;
 int32_t version;
 int32_t hdr_size;
};

void trk_hdr_init(trk_hdr_t *);
trk_hdr_t *trk_get_hdr(tract_file_t *trk);
void trk_hdr_print(FILE *fp, trk_hdr_t *);

tract_file_t *trk_open_read(const char *filename);
tract_file_t *trk_open_write(const char *filename, trk_hdr_t *);

#endif // #ifndef __TRK_H__
