#ifndef __AFFINE_H__
#define __AFFINE_H__

#include "point.h"

typedef struct affine affine_t;
typedef coord_t angle_t;

size_t affine_size(int dim);

// Constructor/destructor
affine_t *affine_create(int dim);
affine_t *affine_create_at(int dim, void *location);
affine_t *affine_copy(affine_t *);
void affine_destroy(affine_t *);

void affine_assign(affine_t *dst, affine_t *src);

// Accesssors
int affine_dim(affine_t *);

coord_t affine_matrix_get(affine_t *, int i, int j);
void affine_matrix_set(affine_t *, int i, int j, coord_t v);

gsl_matrix *affine_as_gsl_matrix(affine_t *A);

// Standard transforms
void affine_identity(affine_t *);
void affine_rotation2(affine_t *, angle_t angle);
void affine_rotation3(affine_t *, angle_t angle, point_t axis);
void affine_translation(affine_t *, point_t translation);
void affine_scale(affine_t *, point_t scale);

// Math
void affine_transform(affine_t *, point_t dst, point_t src);
void affine_compose(affine_t *dst, affine_t *A, affine_t *B);
void affine_inverse_transform(affine_t *, point_t dst, point_t src);

// I/O
void affine_print(affine_t *);
void affine_fprint_fmt(FILE *, affine_t *, const char *coord_fmt);

// Composition shortcuts
void affine_pre_rotate2(affine_t *, angle_t angle);
void affine_post_rotate2(affine_t *, angle_t angle);

void affine_pre_rotate3(affine_t *, angle_t angle, point_t axis);
void affine_post_rotate3(affine_t *, angle_t angle, point_t axis);

void affine_pre_translate(affine_t *, point_t translation);
void affine_post_translate(affine_t *, point_t translation);

void affine_pre_scale(affine_t *, point_t scale);
void affine_post_scale(affine_t *, point_t scale);

#endif // #ifndef __AFFINE_H__
