#ifndef __KD_TREE_H__
#define __KD_TREE_H__


typedef struct spatial_tree_options spatial_tree_options_t;
typedef struct spatial_tree_entry spatial_tree_entry_t;
typedef struct spatial_tree spatial_tree_t;

struct spatial_tree_options {
  size_t max_bin_size;
  size_t dim;

  // TODO: arbitrary split rule
};

struct spatial_tree_entry {
  // Contents
  double *p;
  void *data;

  spatial_tree_t *t;  // The node that this a bin of

  // Doubly linked list for the bin
  spatial_tree_entry_t *prev;
  spatial_tree_entry_t *next;
};

struct spatial_tree
{
  // The bin
  size_t bin_size;
  spatial_tree_entry_t *bin;

  // The split of this node
  int split_dim;
  double split;

  // Tree relationships
  spatial_tree_t *parent;
  int subtree_count;
  spatial_tree_t *subtrees[2];

  spatial_tree_options_t *opts;
};

// Create a new spatial tree
spatial_tree_t *spatial_tree_create(spatial_tree_options_t *opts);

void spatial_tree_destroy(spatial_tree_t *t);

// Adds (p, data) at a leaf node in the spacetree
//   returns: the entry for (p, data)
spatial_tree_entry_t *spatial_tree_add(spatial_tree_t *t, double *p, void *data);

// TODO: adding a group of points, optimally splitting over whole group

// Removes an entry from the spatial tree
void spatial_tree_remove(spatial_tree_entry_t *);

// Find k nearest neighbors
void spatial_tree_nearest_neighbor(
  spatial_tree_t *t, double *p,
	int k, spatial_tree_entry_t **nn);

// Find all neighbors within a given ball of radius distance around p
void spatial_tree_neighborhood(
  spatial_tree_t *t, double *p, double distance,
	void (*add_neighbor)(void *closure_data, spatial_tree_entry_t *neighbor),
	void *closure_data);

#endif // #ifndef __KD_TREE_H__
