#include "gzip.h"
#include "internal.h"

#include <zlib.h>

typedef struct {
  mnemosyne_t base;
	gzFile fp;
} mnemosyne_gzip_t;

static const mnemosyne_functions_t gzip_functions;

static inline mnemosyne_gzip_t *cast(mnemosyne_t *mn)
{
	assert(mn->v == &gzip_functions);
	return (mnemosyne_gzip_t *)mn;
}

mnemosyne_t *mnemosyne_open_gzip(const char *filename, const char *mode,
																 const mnemosyne_options_t *opts)

{
	mnemosyne_gzip_t *mn;

	// Initialize base class
	mn = (mnemosyne_gzip_t *)malloc(sizeof(mnemosyne_gzip_t));
	mn->base.v = &gzip_functions;
	mn->base.opts = *opts;

	// Open file
	mn->fp = gzopen(filename, mode);
	if(mn->fp == NULL) {
		free(mn);
		return NULL;
	}

	assert((intptr_t)&mn->base == (intptr_t)mn);
	return &mn->base;
}

static int gzip_close(mnemosyne_t *mn)
{
	int s;

	s = gzclose(cast(mn)->fp);
	free(mn);

	return s;
}

static bool gzip_eof(mnemosyne_t *mn)
{
	return gzeof(cast(mn)->fp) != 0;
}

static int gzip_err(mnemosyne_t *mn)
{
	int err;

	gzerror(cast(mn)->fp, &err);

	return err;
}

static size_t gzip_write(mnemosyne_t *mn, const void *buffer, size_t size, size_t nitems)
{
	return (size_t)gzwrite(cast(mn)->fp, buffer, (unsigned)(size * nitems));
}

static size_t gzip_read(mnemosyne_t *mn, void *buffer, size_t size, size_t nitems)
{
	return (size_t)gzread(cast(mn)->fp, buffer, (unsigned)(size * nitems));
}

static size_t gzip_printf(mnemosyne_t *mn, const char *format, va_list args)
{
	int len;
	char *buf;

	// Allocate buffer
	// TODO: reset args
	len = vsnprintf(NULL, 0, format, args);
	buf = (char *)malloc(len + 1);

	// Format
	vsnprintf(buf, len, format, args);

	// Write
	len = gzwrite(cast(mn)->fp, buf, (unsigned)len);
	
	// Cleanup
	free(buf);

	return (size_t)len;
}

static const mnemosyne_functions_t gzip_functions = {
	.close = &gzip_close,
	.eof = &gzip_eof,
	.write = &gzip_write,
	.read = &gzip_read,
	.printf = NULL
};
