#ifndef __MNEMOSYNE_INTERNAL_H__
#define __MNEMOSYNE_INTERNAL_H__

#include "mnemosyne.h"

typedef struct {
  int (*close)(mnemosyne_t *);
	bool (*eof)(mnemosyne_t *);
	int (*err)(mnemosyne_t *);

	size_t (*write)(mnemosyne_t *, const void *buffer, size_t size, size_t nitems);	
	size_t (*read)(mnemosyne_t *, void *buffer, size_t size, size_t nitems);	
	size_t (*printf)(mnemosyne_t *, const char *format, va_list args);

} mnemosyne_functions_t;

struct mnemosyne {
	const mnemosyne_functions_t *v;
	mnemosyne_options_t opts;
};

/////////////////////// Calling virtual functions /////////////////

static inline bool mneof(mnemosyne_t *mn)
{
	return (*mn->v->eof)(mn);
}

static inline size_t mnwrite(mnemosyne_t *mn, const void *buffer, size_t size, size_t nitems)
{
	return (*mn->v->write)(mn, buffer, size, nitems);
}

static inline size_t mnread(mnemosyne_t *mn, void *buffer, size_t size, size_t nitems)
{
	return (*mn->v->read)(mn, buffer, size, nitems);
}

static inline size_t mnvprintf(mnemosyne_t *mn, const char *format, va_list args)
{
	return (*mn->v->printf)(mn, format, args);
}

static inline size_t mnprintf(mnemosyne_t *mn, const char *format, ...)
{
	va_list args;
	size_t n;

	va_start(args, format);
	n = (*mn->v->printf)(mn, format, args);
	va_end(args);

	return n;
}

/////////////////////// Utility Functions /////////////////


#endif // #ifndef __MNEMOSYNE_INTERNAL_H__
