#include "plain.h"
#include "internal.h"

typedef struct {
  mnemosyne_t base;
	FILE *fp;
} mnemosyne_plain_t;

static const mnemosyne_functions_t plain_functions;

static inline mnemosyne_plain_t *cast(mnemosyne_t *mn)
{
	assert(mn->v == &plain_functions);
	return (mnemosyne_plain_t *)mn;
}

mnemosyne_t *mnemosyne_open_plain(const char *filename, const char *mode,
																	const mnemosyne_options_t *opts)
{
	mnemosyne_plain_t *mn;

	// Initialize base class
	mn = (mnemosyne_plain_t *)malloc(sizeof(mnemosyne_plain_t));
	mn->base.v = &plain_functions;
	mn->base.opts = *opts;

	// Open file
	mn->fp = fopen(filename, mode);
	if(mn->fp == NULL) {
		free(mn);
		return NULL;
	}

	assert((intptr_t)&mn->base == (intptr_t)mn);
	return &mn->base;
}

static int plain_close(mnemosyne_t *mn)
{
	int s;
	
	s = fclose(cast(mn)->fp);
	free(mn);

	return s;
}

static bool plain_eof(mnemosyne_t *mn)
{
	return feof(cast(mn)->fp) != 0;
}

static int plain_err(mnemosyne_t *mn)
{
	return ferror(cast(mn)->fp);
}

static size_t plain_write(mnemosyne_t *mn, const void *buffer, size_t size, size_t nitems)
{
	return fwrite(buffer, size, nitems, cast(mn)->fp);
}

static size_t plain_read(mnemosyne_t *mn, void *buffer, size_t size, size_t nitems)
{
	return fread(buffer, size, nitems, cast(mn)->fp);
}

static size_t plain_printf(mnemosyne_t *mn, const char *format, va_list args)
{
	int written;

	written = vfprintf(cast(mn)->fp, format, args);

	return (size_t)written;
}

static const mnemosyne_functions_t plain_functions = {
	.close = &plain_close,
	.eof = &plain_eof,
	.err = &plain_err,
	.write = &plain_write,
	.read = &plain_read,
	.printf = &plain_printf
};
