#include "mnemosyne.h"
#include "util/format.h"

const char * const moby_dick = "\
Call me Ishmael. Some years ago--never mind how long precisely--having\n\
little or no money in my purse, and nothing particular to interest me on\n\
shore, I thought I would sail about a little and see the watery part of\n\
the world. It is a way I have of driving off the spleen and regulating\n\
the circulation. Whenever I find myself growing grim about the mouth;\n\
whenever it is a damp, drizzly November in my soul; whenever I find\n\
myself involuntarily pausing before coffin warehouses, and bringing up\n\
the rear of every funeral I meet; and especially whenever my hypos get\n\
such an upper hand of me, that it requires a strong moral principle to\n\
prevent me from deliberately stepping into the street, and methodically\n\
knocking people's hats off--then, I account it high time to get to sea\n\
as soon as I can. This is my substitute for pistol and ball. With a\n\
philosophical flourish Cato throws himself upon his sword; I quietly\n\
take to the ship. There is nothing surprising in this. If they but knew\n\
it, almost all men in their degree, some time or other, cherish very\n\
nearly the same feelings towards the ocean with me.\n\
";

int main(int argc, char **argv)
{
	const char *base_filename;
	char *outfile;
	mnemosyne_options_t opts;
	mnemosyne_t *mn;

	// Get output filename
	if(argc >= 2) {
		base_filename = argv[1];
	}
	else {
		base_filename = "mnemosyne-test.txt";
	}

	opts = mnemosyne_default_options;
	if(argc < 3) {
		opts.compress = MNEMOSYNE_COMPRESS_NONE;
	}
	else if(strcmp(argv[2], "gzip") == 0) {
		opts.compress = MNEMOSYNE_COMPRESS_GZIP;
	}
	else if(strcmp(argv[2], "bzip2") == 0) {
		opts.compress = MNEMOSYNE_COMPRESS_BZIP2;
	}

	// Open test file
	outfile = strformat("%s%s", base_filename, mnemosyne_compression_suffix(&opts));
	mn = mnemosyne_open(outfile, "w+", &opts);

	// Write test data
	mnemosyne_write(mn, moby_dick, 1, strlen(moby_dick));

	mnemosyne_printf(mn, "%d %f %s\n", 5, 18.23, "which");

	// Cleanup
	mnemosyne_close(mn);
	free(outfile);

	return 0;
}
