#include "parallel.h"

#include <pthread.h>

typedef struct {
	pthread_mutex_t lock;

	int job_i, job_count;

  int (*closure)(int job_i, void *closure_data);
  void *closure_data;

} jobs_t;

static int job_get(jobs_t *jobs)
{
	int i;

	pthread_mutex_lock(&jobs->lock);
	i = (jobs->job_i)++;
	pthread_mutex_unlock(&jobs->lock);

	return i;
}

static void *worker_routine(void *arg)
{
	jobs_t *jobs;
	int i;

	jobs = (jobs_t *)arg;

	while((i = job_get(jobs)) < jobs->job_count) {
		if(jobs->closure(i, jobs->closure_data) != 0) {
			// TODO: error
		}
	}

	return jobs;
}

int parallel_execute(int job_count, int worker_count,
										 int (*closure)(int job_i, void *closure_data),
										 void *closure_data)
{
	jobs_t jobs;
	int j;

	// Setup
  pthread_mutex_init(&jobs.lock, NULL);
	
  jobs.job_i = 0;
  jobs.job_count = job_count;

	jobs.closure = closure;
	jobs.closure_data = closure_data;

	if(worker_count < 0) {
		worker_count = DEFAULT_THREAD_COUNT;
	}
	if(job_count < worker_count) {
		worker_count = job_count;
	}
	pthread_t workers[worker_count];

	// Spawn worker threads
  for(j = 0; j < worker_count; j++) {
    pthread_create(workers + j, NULL, &worker_routine, &jobs);
  }

  // Wait for completion
  for(j = 0; j < worker_count; j++) {
    pthread_join(workers[j], NULL);
  }

  // Cleanup
  pthread_mutex_destroy(&jobs.lock);

	return 1;
}
