#ifndef __PATH_H__
#define __PATH_H__

#include "point.h"

typedef struct path path_t;

// Constructor/destructor
path_t *path_create(int dim, int length);
path_t *path_copy(path_t *);
void path_destroy(path_t *);

path_t *path_resize(path_t *, int new_length);
path_t *path_append(path_t *, point_t);

// Accessors
point_t path_get(path_t *, int i);
int path_length(path_t *);
int path_dim(path_t *);

// Modifiers
void path_zero(path_t *);

/**
 * Reverse the order of the points in the path.
 */
void path_reverse(path_t *);

/**
 * Append one path onto another, returning a new path
 */
path_t *path_add(path_t *start, path_t *end);

// Calculations
length_t path_l2_length(path_t *);

/**
 * Estimate the tangent to the curve at a point.
 *
 * A central difference where possible, forward and backward difference
 * at the respective endpoints.
 */
void path_tangent(path_t *, int i, point_t d);

/**
 * Estimate the normal vector.
 *
 * A central difference where possible, forward and backward difference
 * at the respective endpoints.
 *
 * @see Bradie p445
 */
void path_normal(path_t *, int i, point_t dd);

#endif // #ifndef __PATH_H__
