#ifndef __POINT_H__
#define __POINT_H__

typedef double coord_t;
typedef coord_t length_t;
typedef coord_t *point_t;

static inline void point_zero(int dim, point_t p)
{
  int i;

  for(i = 0; i < dim; i++) {
    p[i] = 0;
  }
}

static inline void point_assign(int dim, point_t dst, point_t src)
{
  int d;

  for(d = 0; d < dim; d++) {
    dst[d] = src[d];
  }
}

// Norms
length_t point_l1_norm(int dim, point_t p);
length_t point_l2_norm(int dim, point_t p);
length_t point_linfty_norm(int dim, point_t p);
length_t point_squared_l2_norm(int dim, point_t p);

length_t point_l1_dist(int dim, point_t p, point_t q);
length_t point_l2_dist(int dim, point_t p, point_t q);
length_t point_linfty_dist(int dim, point_t p, point_t q);

// Math
coord_t point_dot_product(int dim, point_t p, point_t q);
void point_projection(int dim, point_t from, point_t to, point_t proj);

static inline void point_mul(int dim, point_t p, coord_t x)
{
  int d;

  for(d = 0; d < dim; d++) {
    p[d] *= x;
  }
}

static inline void point_add(int dim, point_t dst, point_t a, point_t b)
{
  int d;

  for(d = 0; d < dim; d++) {
    dst[d] = a[d] + b[d];
  }
}

static inline void point_sub(int dim, point_t dst, point_t a, point_t b)
{
  int d;

  for(d = 0; d < dim; d++) {
    dst[d] = a[d] - b[d];
  }
}

void unit_vector(int dim, point_t v, point_t unit);

// I/O
void point_printf(int dim, point_t p);
void point_fprintf(FILE *, int dim, point_t p);
int point_snprintf(size_t buflen, char *buf, int dim, point_t p);

void point_printf_fmt(int dim, point_t p, const char *coord_fmt);
void point_fprintf_fmt(FILE *, int dim, point_t p, const char *coord_fmt);
int point_snprintf_fmt(
  size_t buflen, char *buf,
  int dim, point_t p,
  const char *coord_fmt);

#define UNSUPPORTED_DIMENSIONALITY(dim)                                 \
  fprintf(stderr, "Unsupported dimensionality %d (%s:%d)\n", (dim), __FILE__, __LINE__); \
  exit(1);

#endif // #ifndef __POINT_H__
